#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_layer3_virtual_networks_v1
short_description: Resource module for Sda Layer3 Virtual Networks V1
description:
- Manage operations create, update and delete of the resource Sda Layer3 Virtual Networks V1.
- Adds layer 3 virtual networks based on user input.
- Deletes layer 3 virtual networks based on user input.
- Updates layer 3 virtual networks based on user input.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Sda Layer3 Virtual Networks's payload.
    elements: dict
    suboptions:
      anchoredSiteId:
        description: Fabric ID of the fabric site this layer 3 virtual network is to
          be anchored at.
        type: str
      fabricIds:
        description: IDs of the fabrics this layer 3 virtual network is to be assigned
          to.
        elements: str
        type: list
      virtualNetworkName:
        description: Name of the layer 3 virtual network.
        type: str
    type: list
  virtualNetworkName:
    description: VirtualNetworkName query parameter. Name of the layer 3 virtual network.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddLayer3VirtualNetworksV1
  description: Complete reference of the AddLayer3VirtualNetworksV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-layer-3-virtual-networks
- name: Cisco DNA Center documentation for SDA DeleteLayer3VirtualNetworksV1
  description: Complete reference of the DeleteLayer3VirtualNetworksV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-layer-3-virtual-networks
- name: Cisco DNA Center documentation for SDA UpdateLayer3VirtualNetworksV1
  description: Complete reference of the UpdateLayer3VirtualNetworksV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-layer-3-virtual-networks
notes:
  - SDK Method used are
    sda.Sda.add_layer3_virtual_networks_v1,
    sda.Sda.delete_layer3_virtual_networks_v1,
    sda.Sda.update_layer3_virtual_networks_v1,

  - Paths used are
    post /dna/intent/api/v1/sda/layer3VirtualNetworks,
    delete /dna/intent/api/v1/sda/layer3VirtualNetworks,
    put /dna/intent/api/v1/sda/layer3VirtualNetworks,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.sda_layer3_virtual_networks_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - anchoredSiteId: string
      fabricIds:
      - string
      virtualNetworkName: string

- name: Delete all
  cisco.dnac.sda_layer3_virtual_networks_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    virtualNetworkName: string

- name: Update all
  cisco.dnac.sda_layer3_virtual_networks_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - anchoredSiteId: string
      fabricIds:
      - string
      id: string
      virtualNetworkName: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#
# Copyright 2018 Red Hat Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible.utils.display import Display

from ansible_collections.ansible.netcommon.plugins.action.network import (
    ActionModule as ActionNetworkModule,
)


display = Display()


class ActionModule(ActionNetworkModule):
    def run(self, tmp=None, task_vars=None):
        module_name = self._task.action.split(".")[-1]
        self._config_module = True if module_name == "netconf_config" else False
        persistent_connection = self._play_context.connection.split(".")[-1]
        warnings = []

        if persistent_connection != "netconf":
            return {
                "failed": True,
                "msg": "Connection type %s is not valid for %s module. "
                "Valid connection type is netconf." % (self._play_context.connection, module_name),
            }

        result = super(ActionModule, self).run(task_vars=task_vars)
        if warnings:
            if "warnings" in result:
                result["warnings"].extend(warnings)
            else:
                result["warnings"] = warnings
        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_device_update_role_v1
short_description: Resource module for Network Device Update Role V1
description:
- Manage operation update of the resource Network Device Update Role V1.
- Updates the role of the device as access, core, distribution, border router.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: DeviceId of the Device.
    type: str
  role:
    description: Role of device as ACCESS, CORE, DISTRIBUTION, BORDER ROUTER.
    type: str
  roleSource:
    description: Role source as MANUAL / AUTO.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices UpdateDeviceRoleV1
  description: Complete reference of the UpdateDeviceRoleV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-device-role
notes:
  - SDK Method used are
    devices.Devices.update_device_role_v1,

  - Paths used are
    put /dna/intent/api/v1/network-device/brief,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.network_device_update_role_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    id: string
    role: string
    roleSource: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: images_info
short_description: Information module for Images Info
description:
- This module represents an alias of the module images_v1_info
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  siteId:
    description:
    - >
      SiteId query parameter. Site identifier to get the list of all available products under the site. The
      default value is the global site. See https //developer.cisco.com/docs/dna-center/get-site for `siteId`.
    type: str
  productNameOrdinal:
    description:
    - >
      ProductNameOrdinal query parameter. The product name ordinal is a unique value for each network device
      product. The productNameOrdinal can be obtained from the response of API
      `/dna/intent/api/v1/siteWiseProductNames`.
    type: float
  supervisorProductNameOrdinal:
    description:
    - >
      SupervisorProductNameOrdinal query parameter. The supervisor engine module ordinal is a unique value for
      each supervisor module. The `supervisorProductNameOrdinal` can be obtained from the response of API
      `/dna/intent/api/v1/siteWiseProductNames`.
    type: float
  imported:
    description:
    - >
      Imported query parameter. When the value is set to `true`, it will include physically imported images.
      Conversely, when the value is set to `false`, it will include image records from the cloud. The identifier
      for cloud images can be utilized to download images from Cisco.com to the disk.
    type: bool
  name:
    description:
    - >
      Name query parameter. Filter with software image or add-on name. Supports partial case-insensitive search. A
      minimum of 3 characters is required for the search.
    type: str
  version:
    description:
    - >
      Version query parameter. Filter with image version. Supports partial case-insensitive search. A minimum of 3
      characters is required for the search.
    type: str
  golden:
    description:
    - >
      Golden query parameter. When set to `true`, it will retrieve the images marked as tagged golden. When set to
      `false`, it will retrieve the images marked as not tagged golden.
    type: bool
  integrity:
    description:
    - >
      Integrity query parameter. Filter with verified images using Integrity Verification Available values
      UNKNOWN, VERIFIED.
    type: str
  hasAddonImages:
    description:
    - >
      HasAddonImages query parameter. When set to `true`, it will retrieve the images which have add-on images.
      When set to `false`, it will retrieve the images which do not have add-on images.
    type: bool
  isAddonImages:
    description:
    - >
      IsAddonImages query parameter. When set to `true`, it will retrieve the images that an add-on image. When
      set to `false`, it will retrieve the images that are not add-on images.
    type: bool
  offset:
    description:
    - >
      Offset query parameter. The first record to show for this page; the first record is numbered 1. The minimum
      value is 1.
    type: float
  limit:
    description:
    - >
      Limit query parameter. The number of records to show for this page. The minimum and maximum values are 1 and
      500, respectively.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Software Image Management (SWIM) ReturnsListOfSoftwareImagesV1
  description: Complete reference of the ReturnsListOfSoftwareImagesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!returns-list-of-software-images
notes:
  - SDK Method used are
    software_image_management_swim.SoftwareImageManagementSwim.returns_list_of_software_images_v1,

  - Paths used are
    get /dna/intent/api/v1/images,
  - It should be noted that this module is an alias of images_v1_info

"""

EXAMPLES = r"""
- name: Get all Images Info
  cisco.dnac.images_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    siteId: string
    productNameOrdinal: 0
    supervisorProductNameOrdinal: 0
    imported: True
    name: string
    version: string
    golden: True
    integrity: string
    hasAddonImages: True
    isAddonImages: True
    offset: 0
    limit: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "imported": true,
          "name": "string",
          "version": "string",
          "imageType": "string",
          "recommended": "string",
          "ciscoLatest": true,
          "integrityStatus": "string",
          "isAddonImage": true,
          "hasAddonImages": true,
          "goldenTaggingDetails": [
            {
              "deviceRoles": [
                "string"
              ],
              "deviceTags": [
                "string"
              ],
              "inheritedSiteId": "string",
              "inheritedSiteName": "string"
            }
          ],
          "productNames": [
            {
              "id": "string",
              "productName": "string",
              "productNameOrdinal": 0,
              "supervisorProductName": "string",
              "supervisorProductNameOrdinal": 0
            }
          ],
          "isGoldenTagged": true
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: qos_device_interface
short_description: Resource module for Qos Device Interface
description:
- This module represents an alias of the module qos_device_interface_v1
version_added: '4.0.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Id of the qos device info, this object holds all
      qos device interface infos associate with network device id.
    type: str
  payload:
    description: Qos Device Interface's payload.
    elements: dict
    suboptions:
      excludedInterfaces:
        description: Excluded interfaces ids.
        elements: str
        type: list
      id:
        description: Id of Qos device info.
        type: str
      name:
        description: Device name.
        type: str
      networkDeviceId:
        description: Network device id.
        type: str
      qosDeviceInterfaceInfo:
        description: Qos Device Interface's qosDeviceInterfaceInfo.
        elements: dict
        suboptions:
          dmvpnRemoteSitesBw:
            description: Dmvpn remote sites bandwidth.
            elements: int
            type: list
          instanceId:
            description: Instance id.
            type: int
          interfaceId:
            description: Interface id.
            type: str
          interfaceName:
            description: Interface name.
            type: str
          label:
            description: SP Profile name.
            type: str
          role:
            description: Interface role.
            type: str
          uploadBW:
            description: Upload bandwidth.
            type: int
        type: list
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Application Policy CreateQosDeviceInterfaceInfoV1
  description: Complete reference of the CreateQosDeviceInterfaceInfoV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-qos-device-interface-info
- name: Cisco DNA Center documentation for Application Policy DeleteQosDeviceInterfaceInfoV1
  description: Complete reference of the DeleteQosDeviceInterfaceInfoV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-qos-device-interface-info
- name: Cisco DNA Center documentation for Application Policy UpdateQosDeviceInterfaceInfoV1
  description: Complete reference of the UpdateQosDeviceInterfaceInfoV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-qos-device-interface-info
notes:
  - SDK Method used are
    application_policy.ApplicationPolicy.create_qos_device_interface_info_v1,
    application_policy.ApplicationPolicy.delete_qos_device_interface_info_v1,
    application_policy.ApplicationPolicy.update_qos_device_interface_info_v1,

  - Paths used are
    post /dna/intent/api/v1/qos-device-interface-info,
    delete /dna/intent/api/v1/qos-device-interface-info/{id},
    put /dna/intent/api/v1/qos-device-interface-info,
  - It should be noted that this module is an alias of qos_device_interface_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.qos_device_interface:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - excludedInterfaces:
      - string
      id: string
      name: string
      networkDeviceId: string
      qosDeviceInterfaceInfo:
      - dmvpnRemoteSitesBw:
        - 0
        instanceId: 0
        interfaceId: string
        interfaceName: string
        label: string
        role: string
        uploadBW: 0

- name: Create
  cisco.dnac.qos_device_interface:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - excludedInterfaces:
      - string
      name: string
      networkDeviceId: string
      qosDeviceInterfaceInfo:
      - dmvpnRemoteSitesBw:
        - 0
        interfaceId: string
        interfaceName: string
        label: string
        role: string
        uploadBW: 0

- name: Delete by id
  cisco.dnac.qos_device_interface:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_security_group_info
version_added: 1.0.0
short_description: Gather information about EC2 security groups in AWS
description:
    - Gather information about EC2 security groups in AWS.
author:
- Henrique Rodrigues (@Sodki)
options:
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value. See
        U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeSecurityGroups.html) for
        possible filters. Filter names and values are case sensitive. You can also use underscores (_)
        instead of dashes (-) in the filter keys, which will take precedence in case of conflict.
    required: false
    default: {}
    type: dict
notes:
  - By default, the module will return all security groups in a region. To limit results use the
    appropriate filters.
  - Prior to release 5.0.0 this module was called M(amazon.aws.ec2_group_info). The usage did not
    change.

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Gather information about all security groups
- amazon.aws.ec2_security_group_info:

# Gather information about all security groups in a specific VPC
- amazon.aws.ec2_security_group_info:
    filters:
      vpc-id: vpc-12345678

# Gather information about all security groups in a specific VPC
- amazon.aws.ec2_security_group_info:
    filters:
      vpc-id: vpc-12345678

# Gather information about a security group
- amazon.aws.ec2_security_group_info:
    filters:
      group-name: example-1

# Gather information about a security group by id
- amazon.aws.ec2_security_group_info:
    filters:
      group-id: sg-12345678

# Gather information about a security group with multiple filters, also mixing the use of underscores as filter keys
- amazon.aws.ec2_security_group_info:
    filters:
      group_id: sg-12345678
      vpc-id: vpc-12345678

# Gather information about various security groups
- amazon.aws.ec2_security_group_info:
    filters:
      group-name:
        - example-1
        - example-2
        - example-3

# Gather information about any security group with a tag key Name and value Example.
# The quotes around 'tag:name' are important because of the colon in the value
- amazon.aws.ec2_security_group_info:
    filters:
      "tag:Name": Example
"""

RETURN = r"""
security_groups:
    description: Security groups that match the provided filters. Each element consists of a dict with all the information related to that security group.
    type: list
    returned: always
    elements: dict
    contains:
        description:
            description: The description of the security group.
            returned: always
            type: str
        group_id:
            description: The ID of the security group.
            returned: always
            type: str
        group_name:
            description: The name of the security group.
            returned: always
            type: str
        ip_permissions:
            description: The inbound rules associated with the security group.
            returned: always
            type: list
            elements: dict
            contains:
                from_port:
                    description: If the protocol is TCP or UDP, this is the start of the port range.
                    type: int
                    sample: 80
                ip_protocol:
                    description: The IP protocol name or number.
                    returned: always
                    type: str
                ip_ranges:
                    description: The IPv4 ranges.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        cidr_ip:
                            description: The IPv4 CIDR range.
                            returned: always
                            type: str
                ipv6_ranges:
                    description: The IPv6 ranges.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        cidr_ipv6:
                            description: The IPv6 CIDR range.
                            returned: always
                            type: str
                prefix_list_ids:
                    description: The prefix list IDs.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        prefix_list_id:
                            description: The ID of the prefix.
                            returned: always
                            type: str
                to_group:
                    description: If the protocol is TCP or UDP, this is the end of the port range.
                    type: int
                    sample: 80
                user_id_group_pairs:
                    description: The security group and AWS account ID pairs.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        group_id:
                            description: The security group ID of the pair.
                            returned: always
                            type: str
                        user_id:
                            description: The user ID of the pair.
                            returned: always
                            type: str
        ip_permissions_egress:
            description: The outbound rules associated with the security group.
            returned: always
            type: list
            elements: dict
            contains:
                ip_protocol:
                    description: The IP protocol name or number.
                    returned: always
                    type: str
                ip_ranges:
                    description: The IPv4 ranges.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        cidr_ip:
                            description: The IPv4 CIDR range.
                            returned: always
                            type: str
                ipv6_ranges:
                    description: The IPv6 ranges.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        cidr_ipv6:
                            description: The IPv6 CIDR range.
                            returned: always
                            type: str
                prefix_list_ids:
                    description: The prefix list IDs.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        prefix_list_id:
                            description: The ID of the prefix.
                            returned: always
                            type: str
                user_id_group_pairs:
                    description: The security group and AWS account ID pairs.
                    returned: always
                    type: list
                    elements: dict
                    contains:
                        group_id:
                            description: The security group ID of the pair.
                            returned: always
                            type: str
                        user_id:
                            description: The user ID of the pair.
                            returned: always
                            type: str
        owner_id:
            description: The AWS account ID of the owner of the security group.
            returned: always
            type: str
        tags:
            description: The tags associated with the security group.
            returned: always
            type: dict
        vpc_id:
            description: The ID of the VPC for the security group.
            returned: always
            type: str
    sample: [
        {
            "description": "created by rds_instance integration tests",
            "group_id": "sg-036496a610b79da88",
            "group_name": "ansible-test-89355088-unknown5c5f67f3ad09-sg-1",
            "ip_permissions": [],
            "ip_permissions_egress": [
                {
                    "ip_protocol": "-1",
                    "ip_ranges": [
                        {
                            "cidr_ip": "0.0.0.0/0"
                        }
                    ],
                    "ipv6_ranges": [],
                    "prefix_list_ids": [],
                    "user_id_group_pairs": []
                }
            ],
            "owner_id": "123456789012",
            "tags": {},
            "vpc_id": "vpc-0bc3bb03f97405435"
        }
    ]
"""

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_security_groups
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import sanitize_filters_to_boto3_filter_list


def main() -> None:
    argument_spec = dict(filters=dict(default={}, type="dict"))

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    connection = module.client("ec2")

    # Sanitize filters
    sanitized_filters = sanitize_filters_to_boto3_filter_list(module.params.get("filters"), ignore_keys=["tag:"])
    try:
        security_groups = describe_security_groups(connection, Filters=sanitized_filters)
    except AnsibleEC2Error as e:
        module.fail_json_aws(e, msg="Failed to describe security groups")

    snaked_security_groups = []
    for security_group in security_groups:
        # Modify boto3 tags list to be ansible friendly dict
        # but don't camel case tags
        security_group = camel_dict_to_snake_dict(security_group)
        security_group["tags"] = boto3_tag_list_to_ansible_dict(
            security_group.get("tags", {}), tag_name_key_name="key", tag_value_key_name="value"
        )
        snaked_security_groups.append(security_group)

    module.exit_json(security_groups=snaked_security_groups)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: subscriber_bulk
short_description: Resource module for Subscriber Bulk
description:
- Manage operation create of the resource Subscriber Bulk.
version_added: '2.8.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  ItemList:
    description: Subscriber Bulk's ItemList.
    elements: dict
    suboptions:
      enabled:
        description: Subscriber is enabled or not.
        type: bool
      friendlyName:
        description: Friendly name for the subscriber.
        type: str
      id:
        description: Subscriber Bulk's id.
        type: str
      identityGroups:
        description: Identity Group(s). With more than one idGroups it needs to be comma
          seperated.
        type: str
      imeis:
        description: Comma separated IMEIs to be attached to the subscriber.
        type: str
      imsi:
        description: IMSI for Subscriber.
        type: str
      ki:
        description: KI.
        type: str
      opc:
        description: OPC.
        type: str
    type: list
  operation:
    description: Subscriber Bulk's operation.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    subscriber.Subscriber.bulk_subscriber_operation,

  - Paths used are
    post /api/v1/fiveg/subscriber/bulk,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.subscriber_bulk:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    ItemList:
    - enabled: true
      friendlyName: string
      id: string
      identityGroups: string
      imeis: string
      imsi: string
      ki: string
      opc: string
    operation: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = r"""
---
module: fact_diff
short_description: Find the difference between currently set facts
version_added: "1.0.0"
description:
    - Compare two facts or variables and get a diff.
options:
  before:
    description:
      - The first fact to be used in the comparison.
    type: raw
    required: True
  after:
    description:
      - The second fact to be used in the comparison.
    type: raw
    required: True
  plugin:
    description:
      - Configure and specify the diff plugin to use
    type: dict
    default: {}
    suboptions:
      name:
        description:
        - The diff plugin to use, in fully qualified collection name format.
        default: ansible.utils.native
        type: str
      vars:
        description:
        - Parameters passed to the diff plugin.
        type: dict
        default: {}
        suboptions:
          skip_lines:
            description:
              - Skip lines matching these regular expressions.
              - Matches will be removed prior to the diff.
              - If the provided I(before) and I(after) are a string, they will be split.
              - Each entry in each list will be cast to a string for the comparison
            type: list
            elements: str

notes:

author:
- Bradley Thornton (@cidrblock)
"""

EXAMPLES = r"""
- ansible.builtin.set_fact:
    before:
      a:
        b:
          c:
            d:
              - 0
              - 1
    after:
      a:
        b:
          c:
            d:
              - 2
              - 3

- name: Show the difference in json format
  ansible.utils.fact_diff:
    before: "{{ before }}"
    after: "{{ after }}"

# TASK [ansible.utils.fact_diff] **************************************
# --- before
# +++ after
# @@ -3,8 +3,8 @@
#          "b": {
#              "c": {
#                  "d": [
# -                    0,
# -                    1
# +                    2,
# +                    3
#                  ]
#              }
#          }
#
# changed: [localhost]

- name: Show the difference in path format
  ansible.utils.fact_diff:
    before: "{{ before | ansible.utils.to_paths }}"
    after: "{{ after | ansible.utils.to_paths }}"

# TASK [ansible.utils.fact_diff] **************************************
# --- before
# +++ after
# @@ -1,4 +1,4 @@
#  {
# -    "a.b.c.d[0]": 0,
# -    "a.b.c.d[1]": 1
# +    "a.b.c.d[0]": 2,
# +    "a.b.c.d[1]": 3
#  }
#
# changed: [localhost]

- name: Show the difference in yaml format
  ansible.utils.fact_diff:
    before: "{{ before | to_nice_yaml }}"
    after: "{{ after | to_nice_yaml }}"

# TASK [ansible.utils.fact_diff] **************************************
# --- before
# +++ after
# @@ -2,5 +2,5 @@
#      b:
#          c:
#              d:
# -            - 0
# -            - 1
# +            - 2
# +            - 3

# changed: [localhost]


#### Show the difference between complex object using restconf
#  ansible_connection: ansible.netcommon.httpapi
#  ansible_httpapi_use_ssl: True
#  ansible_httpapi_validate_certs: False
#  ansible_network_os: ansible.netcommon.restconf

- name: Get the current interface config prior to changes
  ansible.netcommon.restconf_get:
    content: config
    path: /data/Cisco-NX-OS-device:System/intf-items/phys-items
  register: pre

- name: Update the description of eth1/100
  ansible.utils.update_fact:
    updates:
      - path: "pre['response']['phys-items']['PhysIf-list'][{{ index }}]['descr']"
        value: "Configured by ansible {{ 100 | random }}"
  vars:
    index: "{{ pre['response']['phys-items']['PhysIf-list'] | ansible.utils.index_of('eq', 'eth1/100', 'id') }}"
  register: updated

- name: Apply the configuration
  ansible.netcommon.restconf_config:
    path: 'data/Cisco-NX-OS-device:System/intf-items/'
    content: "{{ updated.pre.response}}"
    method: patch

- name: Get the current interface config after changes
  ansible.netcommon.restconf_get:
    content: config
    path: /data/Cisco-NX-OS-device:System/intf-items/phys-items
  register: post

- name: Show the difference
  ansible.utils.fact_diff:
    before: "{{ pre.response | ansible.utils.to_paths }}"
    after: "{{ post.response | ansible.utils.to_paths }}"

# TASK [ansible.utils.fact_diff] *********************************************
# --- before
# +++ after
# @@ -3604,7 +3604,7 @@
#      "phys-items['PhysIf-list'][37].bw": "0",
#      "phys-items['PhysIf-list'][37].controllerId": "",
#      "phys-items['PhysIf-list'][37].delay": "1",
# -    "phys-items['PhysIf-list'][37].descr": "Configured by ansible 95",
# +    "phys-items['PhysIf-list'][37].descr": "Configured by ansible 20",
#      "phys-items['PhysIf-list'][37].dot1qEtherType": "0x8100",
#      "phys-items['PhysIf-list'][37].duplex": "auto",
#      "phys-items['PhysIf-list'][37].id": "eth1/100",

# changed: [nxos101]
"""


RETURN = """

diff_text:
  description: The diff in text format.
  returned: always
  type: str
diff_lines:
  description: The I(diff_text) split into lines.
  returned: always
  type: list
  elements: str

"""



# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The ios lag_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from itertools import groupby

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.lag_interfaces.lag_interfaces import (
    Lag_InterfacesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.lag_interfaces import (
    Lag_interfacesTemplate,
)


class Lag_interfacesFacts(object):
    """The ios lag_interfaces facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Lag_InterfacesArgs.argument_spec

    def get_lag_interfaces_data(self, connection):
        return connection.get("show running-config | section ^interface")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Lag_interfaces network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_lag_interfaces_data(connection)

        # parse native config using the Lag_interfaces template
        lag_interfaces_parser = Lag_interfacesTemplate(lines=data.splitlines(), module=self._module)
        objs = self.process_facts(list(lag_interfaces_parser.parse().values()))
        ansible_facts["ansible_network_resources"].pop("lag_interfaces", None)

        params = utils.remove_empties(
            lag_interfaces_parser.validate_config(
                self.argument_spec,
                {"config": objs},
                redact=True,
            ),
        )

        facts["lag_interfaces"] = params.get("config", [])
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts

    def process_facts(self, all_objs):
        lag_facts = []
        objs = []
        temp_channels = []

        def key_channel(k):
            return k.get("channel")

        for intrf in all_objs:
            if intrf.get("channel"):
                if intrf.get("channel") in temp_channels:
                    temp_channels.remove(intrf.get("channel"))
                objs.append(intrf)
            if "Port-channel" in intrf.get("member"):
                temp_channels.append(intrf.get("member"))

        objs = sorted(objs, key=key_channel)

        for empty_channel in temp_channels:
            objs.append({"channel": empty_channel})

        for key, value in groupby(objs, key_channel):
            if key:
                _v = list(value)
                for v in _v:
                    v.pop("channel")
                lag_facts.append({"name": key, "members": _v})

        return lag_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: allowed_protocols
short_description: Resource module for Allowed Protocols
description:
- Manage operations create, update and delete of the resource Allowed Protocols.
- This API creates an allowed protocol.
- This API deletes an allowed protocol.
- This API allows the client to update an allowed protocol.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  allowChap:
    description: AllowChap flag.
    type: bool
  allowEapFast:
    description: AllowEapFast flag.
    type: bool
  allowEapMd5:
    description: AllowEapMd5 flag.
    type: bool
  allowEapTls:
    description: AllowEapTls flag.
    type: bool
  allowEapTtls:
    description: AllowEapTtls flag.
    type: bool
  allowLeap:
    description: AllowLeap flag.
    type: bool
  allowMsChapV1:
    description: AllowMsChapV1 flag.
    type: bool
  allowMsChapV2:
    description: AllowMsChapV2 flag.
    type: bool
  allowPapAscii:
    description: AllowPapAscii flag.
    type: bool
  allowPeap:
    description: AllowPeap flag.
    type: bool
  allowPreferredEapProtocol:
    description: AllowPreferredEapProtocol flag.
    type: bool
  allowTeap:
    description: AllowTeap flag.
    type: bool
  allowWeakCiphersForEap:
    description: AllowWeakCiphersForEap flag.
    type: bool
  description:
    description: Allowed Protocols's description.
    type: str
  eapFast:
    description: The eapFast is required only if allowEapFast is true, otherwise it
      must be ignored. The object eapFast contains the settings for EAP FAST protocol.
    suboptions:
      allowEapFastEapGtc:
        description: AllowEapFastEapGtc flag.
        type: bool
      allowEapFastEapGtcPwdChange:
        description: The allowEapFastEapGtcPwdChange is required only if allowEapFastEapGtc
          is true, otherwise it must be ignored.
        type: bool
      allowEapFastEapGtcPwdChangeRetries:
        description: The allowEapFastEapGtcPwdChangeRetries is required only if allowEapFastEapGtc
          is true, otherwise it must be ignored. Valid range is 0-3.
        type: int
      allowEapFastEapMsChapV2:
        description: AllowEapFastEapMsChapV2 flag.
        type: bool
      allowEapFastEapMsChapV2PwdChange:
        description: The allowEapFastEapMsChapV2PwdChange is required only if allowEapFastEapMsChapV2
          is true, otherwise it must be ignored.
        type: bool
      allowEapFastEapMsChapV2PwdChangeRetries:
        description: The allowEapFastEapMsChapV2PwdChangeRetries is required only if
          eapTtlsEapMsChapV2 is true, otherwise it must be ignored. Valid range is 0-3.
        type: int
      allowEapFastEapTls:
        description: AllowEapFastEapTls flag.
        type: bool
      allowEapFastEapTlsAuthOfExpiredCerts:
        description: The allowEapFastEapTlsAuthOfExpiredCerts is required only if allowEapFastEapTls
          is true, otherwise it must be ignored.
        type: bool
      eapFastDontUsePacsAcceptClientCert:
        description: The eapFastDontUsePacsAcceptClientCert is required only if eapFastUsePacs
          is FALSE, otherwise it must be ignored.
        type: bool
      eapFastDontUsePacsAllowMachineAuthentication:
        description: The eapFastDontUsePacsAllowMachineAuthentication is required only
          if eapFastUsePacs is FALSE, otherwise it must be ignored.
        type: bool
      eapFastEnableEAPChaining:
        description: EapFastEnableEAPChaining flag.
        type: bool
      eapFastUsePacs:
        description: EapFastUsePacs flag.
        type: bool
      eapFastUsePacsAcceptClientCert:
        description: The eapFastUsePacsAcceptClientCert is required only if eapFastUsePacsAllowAuthenProvisioning
          is true, otherwise it must be ignored.
        type: bool
      eapFastUsePacsAllowAnonymProvisioning:
        description: The eapFastUsePacsAllowAnonymProvisioning is required only if eapFastUsePacs
          is true, otherwise it must be ignored.
        type: bool
      eapFastUsePacsAllowAuthenProvisioning:
        description: The eapFastUsePacsAllowAuthenProvisioning is required only if eapFastUsePacs
          is true, otherwise it must be ignored.
        type: bool
      eapFastUsePacsAllowMachineAuthentication:
        description: EapFastUsePacsAllowMachineAuthentication flag.
        type: bool
      eapFastUsePacsAuthorizationPacTtl:
        description: The eapFastUsePacsAuthorizationPacTtl is required only if eapFastUsePacsStatelessSessionResume
          is true, otherwise it must be ignored.
        type: int
      eapFastUsePacsAuthorizationPacTtlUnits:
        description: The eapFastUsePacsAuthorizationPacTtlUnits is required only if
          eapFastUsePacsStatelessSessionResume is true, otherwise it must be ignored.
          Allowed Values - SECONDS, - MINUTES, - HOURS, - DAYS, - WEEKS.
        type: str
      eapFastUsePacsMachinePacTtl:
        description: The eapFastUsePacsMachinePacTtl is required only if eapFastUsePacsAllowMachineAuthentication
          is true, otherwise it must be ignored.
        type: int
      eapFastUsePacsMachinePacTtlUnits:
        description: The eapFastUsePacsMachinePacTtlUnits is required only if eapFastUsePacsAllowMachineAuthentication
          is true, otherwise it must be ignored. Allowed Values - SECONDS, - MINUTES,
          - HOURS, - DAYS, - WEEKS.
        type: str
      eapFastUsePacsReturnAccessAcceptAfterAuthenticatedProvisioning:
        description: The eapFastUsePacsReturnAccessAcceptAfterAuthenticatedProvisioning
          is required only if eapFastUsePacsAllowAuthenProvisioning is true, otherwise
          it must be ignored.
        type: bool
      eapFastUsePacsStatelessSessionResume:
        description: The eapFastUsePacsStatelessSessionResume is required only if eapFastUsePacs
          is true, otherwise it must be ignored.
        type: bool
      eapFastUsePacsTunnelPacTtl:
        description: The eapFastUsePacsTunnelPacTtl is required only if eapFastUsePacs
          is true, otherwise it must be ignored.
        type: int
      eapFastUsePacsTunnelPacTtlUnits:
        description: The eapFastUsePacsTunnelPacTtlUnits is required only if eapFastUsePacs
          is true, otherwise it must be ignored. Allowed Values - SECONDS, - MINUTES,
          - HOURS, - DAYS, - WEEKS.
        type: str
      eapFastUsePacsUseProactivePacUpdatePrecentage:
        description: The eapFastUsePacsUseProactivePacUpdatePrecentage is required only
          if eapFastUsePacs is true, otherwise it must be ignored.
        type: int
    type: dict
  eapTls:
    description: The eapTls is required only if allowEapTls is true, otherwise it must
      be ignored. The object eapTls contains the settings for EAP TLS protocol.
    suboptions:
      allowEapTlsAuthOfExpiredCerts:
        description: AllowEapTlsAuthOfExpiredCerts flag.
        type: bool
      eapTlsEnableStatelessSessionResume:
        description: EapTlsEnableStatelessSessionResume flag.
        type: bool
      eapTlsSessionTicketPrecentage:
        description: The eapTlsSessionTicketPrecentage is required only if eapTlsEnableStatelessSessionResume
          is true, otherwise it must be ignored.
        type: int
      eapTlsSessionTicketTtl:
        description: Time to live. The eapTlsSessionTicketTtl is required only if eapTlsEnableStatelessSessionResume
          is true, otherwise it must be ignored.
        type: int
      eapTlsSessionTicketTtlUnits:
        description: Time to live time units. The eapTlsSessionTicketTtlUnits is required
          only if eapTlsEnableStatelessSessionResume is true, otherwise it must be ignored.
          Allowed Values - SECONDS, - MINUTES, - HOURS, - DAYS, - WEEKS.
        type: str
    type: dict
  eapTlsLBit:
    description: EapTlsLBit flag.
    type: bool
  eapTtls:
    description: The eapTtls is required only if allowEapTtls is true, otherwise it
      must be ignored. The object eapTtls contains the settings for EAP TTLS protocol.
    suboptions:
      eapTtlsChap:
        description: EapTtlsChap flag.
        type: bool
      eapTtlsEapMd5:
        description: EapTtlsEapMd5 flag.
        type: bool
      eapTtlsEapMsChapV2:
        description: EapTtlsEapMsChapV2 flag.
        type: bool
      eapTtlsEapMsChapV2PwdChange:
        description: The eapTtlsEapMsChapV2PwdChange is required only if eapTtlsEapMsChapV2
          is true, otherwise it must be ignored.
        type: bool
      eapTtlsEapMsChapV2PwdChangeRetries:
        description: The eapTtlsEapMsChapV2PwdChangeRetries is required only if eapTtlsEapMsChapV2
          is true, otherwise it must be ignored. Valid range is 0-3.
        type: int
      eapTtlsMsChapV1:
        description: EapTtlsMsChapV1 flag.
        type: bool
      eapTtlsMsChapV2:
        description: EapTtlsMsChapV2 flag.
        type: bool
      eapTtlsPapAscii:
        description: EapTtlsPapAscii flag.
        type: bool
    type: dict
  id:
    description: Resource UUID, Mandatory for update.
    type: str
  name:
    description: Resource Name.
    type: str
  peap:
    description: Allowed Protocols's peap.
    suboptions:
      allowPeapEapGtc:
        description: AllowPeapEapGtc flag.
        type: bool
      allowPeapEapGtcPwdChange:
        description: The allowPeapEapGtcPwdChange is required only if allowPeapEapGtc
          is true, otherwise it must be ignored.
        type: bool
      allowPeapEapGtcPwdChangeRetries:
        description: The allowPeapEapGtcPwdChangeRetries is required only if allowPeapEapGtc
          is true, otherwise it must be ignored. Valid range is 0-3.
        type: int
      allowPeapEapMsChapV2:
        description: AllowPeapEapMsChapV2 flag.
        type: bool
      allowPeapEapMsChapV2PwdChange:
        description: The allowPeapEapMsChapV2PwdChange is required only if allowPeapEapMsChapV2
          is true, otherwise it must be ignored.
        type: bool
      allowPeapEapMsChapV2PwdChangeRetries:
        description: The allowPeapEapMsChapV2PwdChangeRetries is required only if allowPeapEapMsChapV2
          is true, otherwise it must be ignored. Valid range is 0-3.
        type: int
      allowPeapEapTls:
        description: AllowPeapEapTls flag.
        type: bool
      allowPeapEapTlsAuthOfExpiredCerts:
        description: The allowPeapEapTlsAuthOfExpiredCerts is required only if allowPeapEapTls
          is true, otherwise it must be ignored.
        type: bool
      allowPeapV0:
        description: AllowPeapV0 flag.
        type: bool
      requireCryptobinding:
        description: RequireCryptobinding flag.
        type: bool
    type: dict
  preferredEapProtocol:
    description: The preferredEapProtocol is required only if allowPreferredEapProtocol
      is true, otherwise it must be ignored. Allowed Values - EAP_FAST, - PEAP, - LEAP,
      - EAP_MD5, - EAP_TLS, - EAP_TTLS, - TEAP.
    type: str
  processHostLookup:
    description: ProcessHostLookup flag.
    type: bool
  requireMessageAuth:
    description: RequireMessageAuth flag.
    type: bool
  teap:
    description: The teap is required only if allowTeap is true, otherwise it must be
      ignored. The object teap contains the settings for TEAP protocol.
    suboptions:
      acceptClientCertDuringTunnelEst:
        description: AcceptClientCertDuringTunnelEst flag.
        type: bool
      allowDowngradeMsk:
        description: AllowDowngradeMsk flag.
        type: bool
      allowTeapEapMsChapV2:
        description: AllowTeapEapMsChapV2 flag.
        type: bool
      allowTeapEapMsChapV2PwdChange:
        description: The allowTeapEapMsChapV2PwdChange is required only if allowTeapEapMsChapV2
          is true, otherwise it must be ignored.
        type: bool
      allowTeapEapMsChapV2PwdChangeRetries:
        description: The allowTeapEapMsChapV2PwdChangeRetries is required only if allowTeapEapMsChapV2
          is true, otherwise it must be ignored. Valid range is 0-3.
        type: int
      allowTeapEapTls:
        description: AllowTeapEapTls flag.
        type: bool
      allowTeapEapTlsAuthOfExpiredCerts:
        description: The allowTeapEapTlsAuthOfExpiredCerts is required only if allowTeapEapTls
          is true, otherwise it must be ignored.
        type: bool
      enableEapChaining:
        description: EnableEapChaining flag.
        type: bool
    type: dict
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    allowed_protocols.AllowedProtocols.create_allowed_protocol,
    allowed_protocols.AllowedProtocols.delete_allowed_protocol_by_id,
    allowed_protocols.AllowedProtocols.update_allowed_protocol_by_id,

  - Paths used are
    post /ers/config/allowedprotocols,
    delete /ers/config/allowedprotocols/{id},
    put /ers/config/allowedprotocols/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.ise.allowed_protocols:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    allowChap: true
    allowEapFast: true
    allowEapMd5: true
    allowEapTls: true
    allowEapTtls: true
    allowLeap: true
    allowMsChapV1: true
    allowMsChapV2: true
    allowPapAscii: true
    allowPeap: true
    allowPreferredEapProtocol: true
    allowTeap: true
    allowWeakCiphersForEap: true
    description: string
    eapFast:
      allowEapFastEapGtc: true
      allowEapFastEapGtcPwdChange: true
      allowEapFastEapGtcPwdChangeRetries: 0
      allowEapFastEapMsChapV2: true
      allowEapFastEapMsChapV2PwdChange: true
      allowEapFastEapMsChapV2PwdChangeRetries: 0
      allowEapFastEapTls: true
      allowEapFastEapTlsAuthOfExpiredCerts: true
      eapFastDontUsePacsAcceptClientCert: true
      eapFastDontUsePacsAllowMachineAuthentication: true
      eapFastEnableEAPChaining: true
      eapFastUsePacs: true
      eapFastUsePacsAcceptClientCert: true
      eapFastUsePacsAllowAnonymProvisioning: true
      eapFastUsePacsAllowAuthenProvisioning: true
      eapFastUsePacsAllowMachineAuthentication: true
      eapFastUsePacsAuthorizationPacTtl: 0
      eapFastUsePacsAuthorizationPacTtlUnits: string
      eapFastUsePacsMachinePacTtl: 0
      eapFastUsePacsMachinePacTtlUnits: string
      eapFastUsePacsReturnAccessAcceptAfterAuthenticatedProvisioning: true
      eapFastUsePacsStatelessSessionResume: true
      eapFastUsePacsTunnelPacTtl: 0
      eapFastUsePacsTunnelPacTtlUnits: string
      eapFastUsePacsUseProactivePacUpdatePrecentage: 0
    eapTls:
      allowEapTlsAuthOfExpiredCerts: true
      eapTlsEnableStatelessSessionResume: true
      eapTlsSessionTicketPrecentage: 0
      eapTlsSessionTicketTtl: 0
      eapTlsSessionTicketTtlUnits: string
    eapTlsLBit: true
    eapTtls:
      eapTtlsChap: true
      eapTtlsEapMd5: true
      eapTtlsEapMsChapV2: true
      eapTtlsEapMsChapV2PwdChange: true
      eapTtlsEapMsChapV2PwdChangeRetries: 0
      eapTtlsMsChapV1: true
      eapTtlsMsChapV2: true
      eapTtlsPapAscii: true
    id: string
    name: string
    peap:
      allowPeapEapGtc: true
      allowPeapEapGtcPwdChange: true
      allowPeapEapGtcPwdChangeRetries: 0
      allowPeapEapMsChapV2: true
      allowPeapEapMsChapV2PwdChange: true
      allowPeapEapMsChapV2PwdChangeRetries: 0
      allowPeapEapTls: true
      allowPeapEapTlsAuthOfExpiredCerts: true
      allowPeapV0: true
      requireCryptobinding: true
    preferredEapProtocol: string
    processHostLookup: true
    requireMessageAuth: true
    teap:
      acceptClientCertDuringTunnelEst: true
      allowDowngradeMsk: true
      allowTeapEapMsChapV2: true
      allowTeapEapMsChapV2PwdChange: true
      allowTeapEapMsChapV2PwdChangeRetries: 0
      allowTeapEapTls: true
      allowTeapEapTlsAuthOfExpiredCerts: true
      enableEapChaining: true

- name: Delete by id
  cisco.ise.allowed_protocols:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

- name: Create
  cisco.ise.allowed_protocols:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    allowChap: true
    allowEapFast: true
    allowEapMd5: true
    allowEapTls: true
    allowEapTtls: true
    allowLeap: true
    allowMsChapV1: true
    allowMsChapV2: true
    allowPapAscii: true
    allowPeap: true
    allowPreferredEapProtocol: true
    allowTeap: true
    allowWeakCiphersForEap: true
    description: string
    eapFast:
      allowEapFastEapGtc: true
      allowEapFastEapGtcPwdChange: true
      allowEapFastEapGtcPwdChangeRetries: 0
      allowEapFastEapMsChapV2: true
      allowEapFastEapMsChapV2PwdChange: true
      allowEapFastEapMsChapV2PwdChangeRetries: 0
      allowEapFastEapTls: true
      allowEapFastEapTlsAuthOfExpiredCerts: true
      eapFastDontUsePacsAcceptClientCert: true
      eapFastDontUsePacsAllowMachineAuthentication: true
      eapFastEnableEAPChaining: true
      eapFastUsePacs: true
      eapFastUsePacsAcceptClientCert: true
      eapFastUsePacsAllowAnonymProvisioning: true
      eapFastUsePacsAllowAuthenProvisioning: true
      eapFastUsePacsAllowMachineAuthentication: true
      eapFastUsePacsAuthorizationPacTtl: 0
      eapFastUsePacsAuthorizationPacTtlUnits: string
      eapFastUsePacsMachinePacTtl: 0
      eapFastUsePacsMachinePacTtlUnits: string
      eapFastUsePacsReturnAccessAcceptAfterAuthenticatedProvisioning: true
      eapFastUsePacsStatelessSessionResume: true
      eapFastUsePacsTunnelPacTtl: 0
      eapFastUsePacsTunnelPacTtlUnits: string
      eapFastUsePacsUseProactivePacUpdatePrecentage: 0
    eapTls:
      allowEapTlsAuthOfExpiredCerts: true
      eapTlsEnableStatelessSessionResume: true
      eapTlsSessionTicketPrecentage: 0
      eapTlsSessionTicketTtl: 0
      eapTlsSessionTicketTtlUnits: string
    eapTlsLBit: true
    eapTtls:
      eapTtlsChap: true
      eapTtlsEapMd5: true
      eapTtlsEapMsChapV2: true
      eapTtlsEapMsChapV2PwdChange: true
      eapTtlsEapMsChapV2PwdChangeRetries: 0
      eapTtlsMsChapV1: true
      eapTtlsMsChapV2: true
      eapTtlsPapAscii: true
    name: string
    peap:
      allowPeapEapGtc: true
      allowPeapEapGtcPwdChange: true
      allowPeapEapGtcPwdChangeRetries: 0
      allowPeapEapMsChapV2: true
      allowPeapEapMsChapV2PwdChange: true
      allowPeapEapMsChapV2PwdChangeRetries: 0
      allowPeapEapTls: true
      allowPeapEapTlsAuthOfExpiredCerts: true
      allowPeapV0: true
      requireCryptobinding: true
    preferredEapProtocol: string
    processHostLookup: true
    requireMessageAuth: true
    teap:
      acceptClientCertDuringTunnelEst: true
      allowDowngradeMsk: true
      allowTeapEapMsChapV2: true
      allowTeapEapMsChapV2PwdChange: true
      allowTeapEapMsChapV2PwdChangeRetries: 0
      allowTeapEapTls: true
      allowTeapEapTlsAuthOfExpiredCerts: true
      enableEapChaining: true

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "eapTls": {
        "allowEapTlsAuthOfExpiredCerts": true,
        "eapTlsEnableStatelessSessionResume": true,
        "eapTlsSessionTicketTtl": 0,
        "eapTlsSessionTicketTtlUnits": "string",
        "eapTlsSessionTicketPrecentage": 0
      },
      "peap": {
        "allowPeapEapMsChapV2": true,
        "allowPeapEapMsChapV2PwdChange": true,
        "allowPeapEapMsChapV2PwdChangeRetries": 0,
        "allowPeapEapGtc": true,
        "allowPeapEapGtcPwdChange": true,
        "allowPeapEapGtcPwdChangeRetries": 0,
        "allowPeapEapTls": true,
        "allowPeapEapTlsAuthOfExpiredCerts": true,
        "requireCryptobinding": true,
        "allowPeapV0": true
      },
      "eapFast": {
        "allowEapFastEapMsChapV2": true,
        "allowEapFastEapMsChapV2PwdChange": true,
        "allowEapFastEapMsChapV2PwdChangeRetries": 0,
        "allowEapFastEapGtc": true,
        "allowEapFastEapGtcPwdChange": true,
        "allowEapFastEapGtcPwdChangeRetries": 0,
        "allowEapFastEapTls": true,
        "allowEapFastEapTlsAuthOfExpiredCerts": true,
        "eapFastUsePacs": true,
        "eapFastUsePacsTunnelPacTtl": 0,
        "eapFastUsePacsTunnelPacTtlUnits": "string",
        "eapFastUsePacsUseProactivePacUpdatePrecentage": 0,
        "eapFastUsePacsAllowAnonymProvisioning": true,
        "eapFastUsePacsAllowAuthenProvisioning": true,
        "eapFastUsePacsReturnAccessAcceptAfterAuthenticatedProvisioning": true,
        "eapFastUsePacsAcceptClientCert": true,
        "eapFastUsePacsMachinePacTtl": 0,
        "eapFastUsePacsMachinePacTtlUnits": "string",
        "eapFastUsePacsAllowMachineAuthentication": true,
        "eapFastUsePacsStatelessSessionResume": true,
        "eapFastUsePacsAuthorizationPacTtl": 0,
        "eapFastUsePacsAuthorizationPacTtlUnits": "string",
        "eapFastDontUsePacsAcceptClientCert": true,
        "eapFastDontUsePacsAllowMachineAuthentication": true,
        "eapFastEnableEAPChaining": true
      },
      "eapTtls": {
        "eapTtlsPapAscii": true,
        "eapTtlsChap": true,
        "eapTtlsMsChapV1": true,
        "eapTtlsMsChapV2": true,
        "eapTtlsEapMd5": true,
        "eapTtlsEapMsChapV2": true,
        "eapTtlsEapMsChapV2PwdChange": true,
        "eapTtlsEapMsChapV2PwdChangeRetries": 0
      },
      "teap": {
        "allowTeapEapMsChapV2": true,
        "allowTeapEapMsChapV2PwdChange": true,
        "allowTeapEapMsChapV2PwdChangeRetries": 0,
        "allowTeapEapTls": true,
        "allowTeapEapTlsAuthOfExpiredCerts": true,
        "acceptClientCertDuringTunnelEst": true,
        "enableEapChaining": true,
        "allowDowngradeMsk": true
      },
      "processHostLookup": true,
      "allowPapAscii": true,
      "allowChap": true,
      "allowMsChapV1": true,
      "allowMsChapV2": true,
      "allowEapMd5": true,
      "allowLeap": true,
      "allowEapTls": true,
      "allowEapTtls": true,
      "allowEapFast": true,
      "allowPeap": true,
      "allowTeap": true,
      "allowPreferredEapProtocol": true,
      "preferredEapProtocol": "string",
      "eapTlsLBit": true,
      "allowWeakCiphersForEap": true,
      "requireMessageAuth": true,
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: discovery_range_v1_info
short_description: Information module for Discovery Range V1
description:
- Get all Discovery Range V1.
- Returns the discoveries by specified range.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  startIndex:
    description:
    - StartIndex path parameter. Starting index for the records.
    type: int
  recordsToReturn:
    description:
    - RecordsToReturn path parameter. Number of records to fetch from the starting index.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Discovery GetDiscoveriesByRangeV1
  description: Complete reference of the GetDiscoveriesByRangeV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-discoveries-by-range
notes:
  - SDK Method used are
    discovery.Discovery.get_discoveries_by_range_v1,

  - Paths used are
    get /dna/intent/api/v1/discovery/{startIndex}/{recordsToReturn},

"""

EXAMPLES = r"""
- name: Get all Discovery Range V1
  cisco.dnac.discovery_range_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startIndex: 0
    recordsToReturn: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "attributeInfo": {},
          "cdpLevel": 0,
          "deviceIds": "string",
          "discoveryCondition": "string",
          "discoveryStatus": "string",
          "discoveryType": "string",
          "enablePasswordList": "string",
          "globalCredentialIdList": [
            "string"
          ],
          "httpReadCredential": {
            "comments": "string",
            "credentialType": "string",
            "description": "string",
            "id": "string",
            "instanceTenantId": "string",
            "instanceUuid": "string",
            "password": "string",
            "port": 0,
            "secure": true,
            "username": "string"
          },
          "httpWriteCredential": {
            "comments": "string",
            "credentialType": "string",
            "description": "string",
            "id": "string",
            "instanceTenantId": "string",
            "instanceUuid": "string",
            "password": "string",
            "port": 0,
            "secure": true,
            "username": "string"
          },
          "id": "string",
          "ipAddressList": "string",
          "ipFilterList": "string",
          "isAutoCdp": true,
          "lldpLevel": 0,
          "name": "string",
          "netconfPort": "string",
          "numDevices": 0,
          "parentDiscoveryId": "string",
          "passwordList": "string",
          "preferredMgmtIPMethod": "string",
          "protocolOrder": "string",
          "retryCount": 0,
          "snmpAuthPassphrase": "string",
          "snmpAuthProtocol": "string",
          "snmpMode": "string",
          "snmpPrivPassphrase": "string",
          "snmpPrivProtocol": "string",
          "snmpRoCommunity": "string",
          "snmpRoCommunityDesc": "string",
          "snmpRwCommunity": "string",
          "snmpRwCommunityDesc": "string",
          "snmpUserName": "string",
          "timeOut": 0,
          "updateMgmtIp": true,
          "userNameList": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_resource_smtp
short_description: Manages resource-smtp objects on Checkpoint over Web Services API
description:
  - Manages resource-smtp objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  mail_delivery_server:
    description:
      - Specify the server to which mail is forwarded.
    type: str
  deliver_messages_using_dns_mx_records:
    description:
      - MX record resolving is used to set the destination IP address of the connection.
    type: bool
  check_rulebase_with_new_destination:
    description:
      - The Rule Base will be rechecked with the new resolved IP address for mail delivery.
    type: bool
  notify_sender_on_error:
    description:
      - Enable error mail delivery.
    type: bool
  error_mail_delivery_server:
    description:
      - Error mail delivery happens if the SMTP security server is unable to deliver the message within the abandon time, and Notify Sender on Error
        is checked.
    type: str
  error_deliver_messages_using_dns_mx_records:
    description:
      - MX record resolving will be used to set the source IP address of the connection used to send the error message.
    type: bool
  error_check_rulebase_with_new_destination:
    description:
      - The Rule Base will be rechecked with the new resolved IP address for error mail delivery.
    type: bool
  exception_track:
    description:
      - Determines if an action specified in the Action 2 and CVP categories taken as a result of a resource definition is logged.
    type: str
    choices: ['none', 'exception log', 'exception alert']
  match:
    description:
      - Set the Match properties for the SMTP resource.
    type: dict
    suboptions:
      sender:
        description:
          - Set the Match sender property for the SMTP resource.
        type: str
      recipient:
        description:
          - Set the Match recipient property for the SMTP resource.
        type: str
  action_1:
    description:
      - Use the Rewriting Rules to rewrite Sender and Recipient headers in emails, you can also rewrite other email headers by using the custom header field.
    type: dict
    suboptions:
      sender:
        description:
          - Rewrite Sender header.
        type: dict
        suboptions:
          original:
            description:
              - Original field.
            type: str
          rewritten:
            description:
              - Replacement field.
            type: str
      recipient:
        description:
          - Rewrite Recipient header.
        type: dict
        suboptions:
          original:
            description:
              - Original field.
            type: str
          rewritten:
            description:
              - Replacement field.
            type: str
      custom_field:
        description:
          - The name of the header.
        type: dict
        suboptions:
          original:
            description:
              - Original field.
            type: str
          rewritten:
            description:
              - Replacement field.
            type: str
          field:
            description:
              - The name of the header.
            type: str
  action_2:
    description:
      - Use this window to configure mail inspection for the SMTP Resource.
    type: dict
    suboptions:
      strip_mime_of_type:
        description:
          - Specifies the MIME type to strip from the message.
        type: str
      strip_file_by_name:
        description:
          - Strips file attachments of the specified name from the message.
        type: str
      mail_capacity:
        description:
          - Restrict the size (in kb) of incoming email attachments.
        type: int
      allowed_characters:
        description:
          - The MIME email headers can consist of 8 or 7 bit characters (7 ASCII and 8 for sending Binary characters) in order to encode mail data.
        type: str
        choices: ['8_bit', '7_bit']
      strip_script_tags:
        description:
          - Strip JAVA scripts.
        type: bool
      strip_applet_tags:
        description:
          - Strip JAVA applets.
        type: bool
      strip_activex_tags:
        description:
          - Strip activeX tags.
        type: bool
      strip_ftp_links:
        description:
          - Strip ftp links.
        type: bool
      strip_port_strings:
        description:
          - Strip ports.
        type: bool
  cvp:
    description:
      - Configure CVP inspection on mail messages.
    type: dict
    suboptions:
      enable_cvp:
        description:
          - Select to enable the Content Vectoring Protocol.
        type: bool
      server:
        description:
          - The UID or Name of the CVP server, make sure the CVP server is already be defined as an OPSEC Application.
        type: str
      allowed_to_modify_content:
        description:
          - Configures the CVP server to inspect but not modify content.
        type: bool
      reply_order:
        description:
          - Designates when the CVP server returns data to the Security Gateway security server.
        type: str
        choices: ['return_data_after_content_is_approved', 'return_data_before_content_is_approved']
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-resource-smtp
  cp_mgmt_resource_smtp:
    deliver_messages_using_dns_mx_records: 'true'
    exception_track: exception log
    mail_delivery_server: deliverServer
    match:
      recipient: recipientName
      sender: senderName
    name: newSmtpResource
    state: present

- name: set-resource-smtp
  cp_mgmt_resource_smtp:
    mail_delivery_server: newServer
    name: newSmtpResource
    state: present

- name: delete-resource-smtp
  cp_mgmt_resource_smtp:
    name: newSmtpResource
    state: absent
"""

RETURN = """
cp_mgmt_resource_smtp:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        mail_delivery_server=dict(type='str'),
        deliver_messages_using_dns_mx_records=dict(type='bool'),
        check_rulebase_with_new_destination=dict(type='bool'),
        notify_sender_on_error=dict(type='bool'),
        error_mail_delivery_server=dict(type='str'),
        error_deliver_messages_using_dns_mx_records=dict(type='bool'),
        error_check_rulebase_with_new_destination=dict(type='bool'),
        exception_track=dict(type='str', choices=['none', 'exception log', 'exception alert']),
        match=dict(type='dict', options=dict(
            sender=dict(type='str'),
            recipient=dict(type='str')
        )),
        action_1=dict(type='dict', options=dict(
            sender=dict(type='dict', options=dict(
                original=dict(type='str'),
                rewritten=dict(type='str')
            )),
            recipient=dict(type='dict', options=dict(
                original=dict(type='str'),
                rewritten=dict(type='str')
            )),
            custom_field=dict(type='dict', options=dict(
                original=dict(type='str'),
                rewritten=dict(type='str'),
                field=dict(type='str')
            ))
        )),
        action_2=dict(type='dict', options=dict(
            strip_mime_of_type=dict(type='str'),
            strip_file_by_name=dict(type='str'),
            mail_capacity=dict(type='int'),
            allowed_characters=dict(type='str', choices=['8_bit', '7_bit']),
            strip_script_tags=dict(type='bool'),
            strip_applet_tags=dict(type='bool'),
            strip_activex_tags=dict(type='bool'),
            strip_ftp_links=dict(type='bool'),
            strip_port_strings=dict(type='bool')
        )),
        cvp=dict(type='dict', options=dict(
            enable_cvp=dict(type='bool'),
            server=dict(type='str'),
            allowed_to_modify_content=dict(type='bool'),
            reply_order=dict(type='str', choices=['return_data_after_content_is_approved', 'return_data_before_content_is_approved'])
        )),
        tags=dict(type='list', elements='str'),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'resource-smtp'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: sda_provision_devices
short_description: Resource module for Sda Provision Devices
description:
- This module represents an alias of the module sda_provision_devices_v1
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. ID of the provisioned device.
    type: str
  networkDeviceId:
    description: NetworkDeviceId query parameter. ID of the network device.
    type: str
  payload:
    description: Sda Provision Devices's payload.
    elements: dict
    suboptions:
      networkDeviceId:
        description: ID of network device to be provisioned.
        type: str
      siteId:
        description: ID of the site this network device needs to be provisioned.
        type: str
    type: list
  siteId:
    description: SiteId query parameter. ID of the site hierarchy.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA ProvisionDevicesV1
  description: Complete reference of the ProvisionDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!provision-devices
- name: Cisco DNA Center documentation for SDA DeleteProvisionedDeviceByIdV1
  description: Complete reference of the DeleteProvisionedDeviceByIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-provisioned-device-by-id
- name: Cisco DNA Center documentation for SDA DeleteProvisionedDevicesV1
  description: Complete reference of the DeleteProvisionedDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-provisioned-devices
- name: Cisco DNA Center documentation for SDA ReProvisionDevicesV1
  description: Complete reference of the ReProvisionDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!re-provision-devices
notes:
  - SDK Method used are
    sda.Sda.delete_provisioned_device_by_id_v1,
    sda.Sda.provision_devices_v1,
    sda.Sda.re_provision_devices_v1,

  - Paths used are
    post /dna/intent/api/v1/sda/provisionDevices,
    delete /dna/intent/api/v1/sda/provisionDevices,
    delete /dna/intent/api/v1/sda/provisionDevices/{id},
    put /dna/intent/api/v1/sda/provisionDevices,
  - It should be noted that this module is an alias of sda_provision_devices_v1

"""

EXAMPLES = r"""
- name: Delete all
  cisco.dnac.sda_provision_devices:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    networkDeviceId: string
    siteId: string

- name: Create
  cisco.dnac.sda_provision_devices:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - networkDeviceId: string
      siteId: string

- name: Update all
  cisco.dnac.sda_provision_devices:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - id: string
      networkDeviceId: string
      siteId: string

- name: Delete by id
  cisco.dnac.sda_provision_devices:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    id: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_mobile_access_rule
short_description: Manages mobile-access-rule objects on Checkpoint over Web Services API
description:
  - Manages mobile-access-rule objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  position:
    description:
      - Position in the rulebase.
    type: str
  name:
    description:
      - Object name.
    type: str
    required: True
  user_groups:
    description:
      - User groups that will be associated with the apps - identified by the name or UID.
    type: list
    elements: str
  applications:
    description:
      - Available apps that will be associated with the user groups - identified by the name or UID.
    type: list
    elements: str
  enabled:
    description:
      - Enable/Disable the rule.
    type: bool
  install_on:
    description:
      - Which Gateways identified by the name or UID to install the policy on.
    type: list
    elements: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-mobile-access-rule
  cp_mgmt_mobile_access_rule:
    applications: New Application
    name: Rule 1
    position: 1
    state: present
    user_groups:
    - my_group

- name: set-mobile-access-rule
  cp_mgmt_mobile_access_rule:
    applications:
    - New Application
    - New Application 2
    name: Rule 1
    state: present
    user_groups:
    - my_group

- name: delete-mobile-access-rule
  cp_mgmt_mobile_access_rule:
    name: Rule 1
    state: absent
"""

RETURN = """
cp_mgmt_mobile_access_rule:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call, api_call_for_rule


def main():
    argument_spec = dict(
        position=dict(type='str'),
        name=dict(type='str', required=True),
        user_groups=dict(type='list', elements='str'),
        applications=dict(type='list', elements='str'),
        enabled=dict(type='bool'),
        install_on=dict(type='list', elements='str'),
        tags=dict(type='list', elements='str'),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'mobile-access-rule'

    if module.params["position"] is None:
        result = api_call(module, api_call_object)
    else:
        result = api_call_for_rule(module, api_call_object)

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_keyvaultkey
version_added: "0.1.2"
short_description: Use Azure KeyVault keys
description:
    - Create or delete a key within a given keyvault.
    - By using Key Vault, you can encrypt keys and secrets.
    - Such as authentication keys, storage account keys, data encryption keys, .PFX files, and passwords.
options:
    keyvault_uri:
        description:
            - URI of the keyvault endpoint.
        required: true
        type: str
    key_name:
        description:
            - Name of the keyvault key.
        required: true
        type: str
    key_type:
        description:
            - The type of key to create. For valid values, see JsonWebKeyType. Possible values include EC, EC-HSM, RSA, RSA-HSM, oct
        default: 'RSA'
        type: str
    key_size:
        description:
            - The key size in bits. For example 2048, 3072, or 4096 for RSA.
        type: int
    key_attributes:
        description:
            - The attributes of a key managed by the key vault service.
        type: dict
        suboptions:
            enabled:
                description:
                    - Whether the key is enabled.
                type: bool
            not_before:
                description:
                    - not valid before date in UTC ISO format without the Z at the end
                type: str
            expires:
                description:
                    - not valid after date in UTC ISO format without the Z at the end
                type: str
    curve:
        description:
            - Elliptic curve name. For valid values, see JsonWebKeyCurveName. Possible values include P-256, P-384, P-521, P-256K.
        type: str
    byok_file:
        description:
            - BYOK file.
        type: str
    pem_file:
        description:
            - PEM file.
        type: str
    pem_password:
        description:
            - PEM password.
        type: str
    state:
        description:
            - Assert the state of the key. Use C(present) to create a key and C(absent) to delete a key.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Ian Philpot (@iphilpot)

'''

EXAMPLES = '''
- name: Create a key
  azure_rm_keyvaultkey:
    key_name: MyKey
    keyvault_uri: https://contoso.vault.azure.net/

- name: Delete a key
  azure_rm_keyvaultkey:
    key_name: MyKey
    keyvault_uri: https://contoso.vault.azure.net/
    state: absent
'''

RETURN = '''
state:
    description:
        - Current state of the key.
    returned: success
    type: complex
    contains:
        key_id:
          description:
              - key resource path.
          type: str
          example: https://contoso.vault.azure.net/keys/hello/e924f053839f4431b35bc54393f98423
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.keyvault.keys import KeyClient
    from azure.core.exceptions import ResourceNotFoundError
    from datetime import datetime
except ImportError:
    # This is handled in azure_rm_common
    pass

key_addribute_spec = dict(
    enabled=dict(type='bool', required=False),
    not_before=dict(type='str', no_log=True, required=False),
    expires=dict(type='str', no_log=True, required=False)
)


class AzureRMKeyVaultKey(AzureRMModuleBase):
    ''' Module that creates or deletes keys in Azure KeyVault '''

    def __init__(self):

        self.module_arg_spec = dict(
            key_name=dict(type='str', required=True),
            keyvault_uri=dict(type='str', no_log=True, required=True),
            key_type=dict(type='str', default='RSA'),
            key_size=dict(type='int'),
            key_attributes=dict(type='dict', no_log=True, options=key_addribute_spec),
            curve=dict(type='str'),
            pem_file=dict(type='str'),
            pem_password=dict(type='str', no_log=True),
            byok_file=dict(type='str'),
            state=dict(type='str', default='present', choices=['present', 'absent'])
        )

        self.results = dict(
            changed=False,
            state=dict()
        )

        self.key_name = None
        self.keyvault_uri = None
        self.key_type = None
        self.key_size = None
        self.key_attributes = None
        self.curve = None
        self.pem_file = None
        self.pem_password = None
        self.state = None
        self.client = None
        self.tags = None

        required_if = [
            ('pem_password', 'present', ['pem_file'])
        ]

        super(AzureRMKeyVaultKey, self).__init__(self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 required_if=required_if,
                                                 supports_tags=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        # Create KeyVaultClient
        self.client = self.get_keyvault_client()

        results = dict()
        changed = False

        try:
            results['key_id'] = self.get_key(self.key_name)

            # Key exists and will be deleted
            if self.state == 'absent':
                changed = True

        except ResourceNotFoundError as ec:
            # Key doesn't exist
            if self.state == 'present':
                changed = True
        except Exception as ec:
            self.fail("Find the key vault secret got exception, exception as {0}".format(ec))

        self.results['changed'] = changed
        self.results['state'] = results

        if not self.check_mode:

            # Create key
            if self.state == 'present' and changed:
                results['key_id'] = self.create_key(self.key_name)
                self.results['state'] = results
                self.results['state']['status'] = 'Created'
            # Delete key
            elif self.state == 'absent' and changed:
                results['key_id'] = self.delete_key(self.key_name)
                self.results['state'] = results
                self.results['state']['status'] = 'Deleted'
        else:
            if self.state == 'present' and changed:
                self.results['state']['status'] = 'Created'
            elif self.state == 'absent' and changed:
                self.results['state']['status'] = 'Deleted'

        return self.results

    def get_keyvault_client(self):

        return KeyClient(vault_url=self.keyvault_uri, credential=self.azure_auth.azure_credential_track2)

    def get_key(self, name, version=''):
        ''' Gets an existing key '''
        key_bundle = self.client.get_key(name, version)

        if key_bundle:
            return key_bundle.id

    def create_key(self, name):
        ''' Creates a key '''

        if self.key_attributes is not None:
            k_enabled = self.key_attributes.get('enabled', True)
            k_not_before = self.key_attributes.get('not_before', None)
            k_expires = self.key_attributes.get('expires', None)
            if k_not_before:
                k_not_before = datetime.fromisoformat(k_not_before.replace('Z', '+00:00'))
            if k_expires:
                k_expires = datetime.fromisoformat(k_expires.replace('Z', '+00:00'))
        else:
            k_enabled = True
            k_not_before = None
            k_expires = None

        key_bundle = self.client.create_key(name=name,
                                            key_type=self.key_type,
                                            size=self.key_size,
                                            curve=self.curve,
                                            tags=self.tags,
                                            enabled=k_enabled,
                                            not_before=k_not_before,
                                            expires_on=k_expires)
        return key_bundle._properties._id

    def delete_key(self, name):
        ''' Deletes a key '''
        deleted_key = self.client.begin_delete_key(name)
        result = self.get_poller_result(deleted_key)
        return result.properties._id


def main():
    AzureRMKeyVaultKey()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sites_telemetry_settings_v1
short_description: Resource module for Sites Telemetry Settings V1
description:
- Manage operation update of the resource Sites Telemetry Settings V1.
- >
   Sets telemetry settings for the given site; `null` values indicate that the setting will be inherited from the
   parent site.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  applicationVisibility:
    description: Sites Telemetry Settings's applicationVisibility.
    suboptions:
      collector:
        description: Sites Telemetry Settings's collector.
        suboptions:
          address:
            description: IP Address. If collection type is 'TelemetryBrokerOrUDPDirector',
              this field value is mandatory otherwise it is optional. Examples "250.162.252.170",
              "2001 db8 3c4d 15 1a2f 1a2b".
            type: str
          collectorType:
            description: Collector Type.
            type: str
          port:
            description: Min 1; Max 65535. If collection type is 'TelemetryBrokerOrUDPDirector',
              this field value is mandatory otherwise it is optional.
            type: int
        type: dict
      enableOnWiredAccessDevices:
        description: Enable Netflow Application Telemetry and Controller Based Application
          Recognition (CBAR) by default upon network device site assignment for wired
          access devices.
        type: bool
    type: dict
  id:
    description: Id path parameter. Site Id, retrievable from the `id` attribute in
      `/dna/intent/api/v1/sites`.
    type: str
  snmpTraps:
    description: Sites Telemetry Settings's snmpTraps.
    suboptions:
      externalTrapServers:
        description: External SNMP trap servers. Example "250.162.252.170","2001 db8
          3c4d 15 1a2f 1a2b".
        elements: str
        type: list
      useBuiltinTrapServer:
        description: Enable this server as a destination server for SNMP traps and messages
          from your network.
        type: bool
    type: dict
  syslogs:
    description: Sites Telemetry Settings's syslogs.
    suboptions:
      externalSyslogServers:
        description: External syslog servers. Example "250.162.252.170", "2001 db8 3c4d
          15 1a2f 1a2b".
        elements: str
        type: list
      useBuiltinSyslogServer:
        description: Enable this server as a destination server for syslog messages.
        type: bool
    type: dict
  wiredDataCollection:
    description: Sites Telemetry Settings's wiredDataCollection.
    suboptions:
      enableWiredDataCollectio:
        description: Track the presence, location, and movement of wired endpoints in
          the network. Traffic received from endpoints is used to extract and store
          their identity information (MAC address and IP address). Other features, such
          as IEEE 802.1X, web authentication, Cisco Security Groups (formerly TrustSec),
          SD-Access, and Assurance, depend on this identity information to operate properly.
          Wired Endpoint Data Collection enables Device Tracking policies on devices
          assigned to the Access role in Inventory.
        type: bool
    type: dict
  wirelessTelemetry:
    description: Sites Telemetry Settings's wirelessTelemetry.
    suboptions:
      enableWirelessTelemetry:
        description: Enables Streaming Telemetry on your wireless controllers in order
          to determine the health of your wireless controller, access points and wireless
          clients.
        type: bool
    type: dict
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Network Settings SetTelemetrySettingsForASiteV1
  description: Complete reference of the SetTelemetrySettingsForASiteV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!set-telemetry-settings-for-a-site
notes:
  - SDK Method used are
    network_settings.NetworkSettings.set_telemetry_settings_for_a_site_v1,

  - Paths used are
    put /dna/intent/api/v1/sites/{id}/telemetrySettings,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.sites_telemetry_settings_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    applicationVisibility:
      collector:
        address: string
        collectorType: string
        port: 0
      enableOnWiredAccessDevices: true
    id: string
    snmpTraps:
      externalTrapServers:
      - string
      useBuiltinTrapServer: true
    syslogs:
      externalSyslogServers:
      - string
      useBuiltinSyslogServer: true
    wiredDataCollection:
      enableWiredDataCollectio: true
    wirelessTelemetry:
      enableWirelessTelemetry: true

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# ansible.content_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the documentation in the module file and re-run
# ansible.content_builder commenting out
# the path to external 'docstring' in build.yaml.
#
##############################################

"""
The arg spec for the ios_service module
"""


class ServiceArgs(object):  # pylint: disable=R0903
    """The arg spec for the ios_service module"""

    argument_spec = {
        "config": {
            "options": {
                "call_home": {"type": "bool"},
                "compress_config": {"type": "bool"},
                "config": {"type": "bool"},
                "counters": {"type": "int", "default": 0},
                "dhcp": {"type": "bool", "default": True},
                "disable_ip_fast_frag": {"type": "bool"},
                "exec_callback": {"type": "bool"},
                "exec_wait": {"type": "bool"},
                "hide_telnet_addresses": {"type": "bool"},
                "internal": {"type": "bool"},
                "linenumber": {"type": "bool"},
                "log": {"type": "bool"},
                "log_hidden": {"type": "bool"},
                "nagle": {"type": "bool"},
                "old_slip_prompts": {"type": "bool"},
                "pad": {"type": "bool"},
                "pad_cmns": {"type": "bool"},
                "pad_from_xot": {"type": "bool"},
                "pad_to_xot": {"type": "bool"},
                "password_encryption": {"type": "bool"},
                "password_recovery": {"type": "bool", "default": True},
                "prompt": {"type": "bool", "default": True},
                "private_config_encryption": {"type": "bool"},
                "pt_vty_logging": {"type": "bool"},
                "scripting": {"type": "bool"},
                "sequence_numbers": {"type": "bool"},
                "slave_coredump": {"type": "bool"},
                "slave_log": {"type": "bool", "default": True},
                "tcp_keepalives_in": {"type": "bool"},
                "tcp_keepalives_out": {"type": "bool"},
                "tcp_small_servers": {
                    "options": {
                        "enable": {"type": "bool"},
                        "max_servers": {"type": "int"},
                        "no_limit": {"type": "bool"},
                    },
                    "type": "dict",
                },
                "telnet_zeroidle": {"type": "bool"},
                "timestamps": {
                    "elements": "dict",
                    "options": {
                        "msg": {"choices": ["debug", "log"], "type": "str"},
                        "enable": {"type": "bool"},
                        "timestamp": {
                            "choices": ["datetime", "uptime"],
                            "type": "str",
                        },
                        "datetime_options": {
                            "options": {
                                "localtime": {"type": "bool"},
                                "msec": {"type": "bool"},
                                "show_timezone": {"type": "bool"},
                                "year": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                    },
                    "type": "list",
                },
                "udp_small_servers": {
                    "options": {
                        "enable": {"type": "bool"},
                        "max_servers": {"type": "int"},
                        "no_limit": {"type": "bool"},
                    },
                    "type": "dict",
                },
                "unsupported_transceiver": {"type": "bool"},
            },
            "type": "dict",
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "rendered",
                "parsed",
            ],
            "default": "merged",
            "type": "str",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: authentication_import_certificate_p12
short_description: Resource module for Authentication Import Certificate P12
description:
- This module represents an alias of the module authentication_import_certificate_p12_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  listOfUsers:
    description: ListOfUsers query parameter. Specify whether the certificate will be
      used for controller ("server"), disaster recovery ("ipsec") or both ("server,
      ipsec"). If no value is provided, the default value taken will be "server".
    elements: dict
    suboptions:
      description:
        description: Authentication Import Certificate P12's listOfUsers.
        type: str
    type: list
  p12Password:
    description: P12Password query parameter. The password for PKCS12 certificate bundle.
    type: str
  pkPassword:
    description: PkPassword query parameter. Password for encrypted private key.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Authentication Management ImportCertificateP12V1
  description: Complete reference of the ImportCertificateP12V1 API.
  link: https://developer.cisco.com/docs/dna-center/#!import-certificate-p-12
notes:
  - SDK Method used are
    authentication_management.AuthenticationManagement.import_certificate_p12_v1,

  - Paths used are
    post /dna/intent/api/v1/certificate-p12,
  - It should be noted that this module is an alias of authentication_import_certificate_p12_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.authentication_import_certificate_p12:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    listOfUsers: []
    p12Password: string
    pkPassword: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2016, Thomas Stringer <tomstr@microsoft.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_functionapp
version_added: "0.1.2"
short_description: Manage Azure Function Apps
description:
    - Create, update or delete an Azure Function App.
options:
    resource_group:
        description:
            - Name of resource group.
        required: true
        aliases:
            - resource_group_name
        type: str
    name:
        description:
            - Name of the Azure Function App.
        required: true
        type: str
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    plan:
        description:
            - App service plan.
            - It can be name of existing app service plan in same resource group as function app.
            - It can be resource id of existing app service plan.
            - Resource id. For example /subscriptions/<subs_id>/resourceGroups/<resource_group>/providers/Microsoft.Web/serverFarms/<plan_name>.
            - It can be a dict which contains C(name), C(resource_group).
            - C(name). Name of app service plan.
            - C(resource_group). Resource group name of app service plan.
        type: raw
    container_settings:
        description:
            - Web app container settings.
        type: dict
        suboptions:
            name:
                description:
                    - Name of container. For example "imagename:tag".
                required: True
                type: str
            registry_server_url:
                description:
                    - Container registry server url. For example C(mydockerregistry.io).
                type: str
            registry_server_user:
                description:
                    - The container registry server user name.
                type: str
            registry_server_password:
                description:
                    - The container registry server password.
                type: str
    storage_account:
        description:
            - Name of the storage account to use.
            - Required when creating.
        type: str
        aliases:
            - storage
            - storage_account_name
    app_settings:
        description:
            - Dictionary containing application settings.
        type: dict
    state:
        description:
            - Assert the state of the Function App. Use C(present) to create or update a Function App and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
    - azure.azcollection.azure_identity_multiple

author:
    - Thomas Stringer (@trstringer)
'''

EXAMPLES = '''
- name: Create a function app
  azure_rm_functionapp:
    resource_group: myResourceGroup
    name: myFunctionApp
    storage_account: myStorageAccount

- name: Create a function app with app settings
  azure_rm_functionapp:
    resource_group: myResourceGroup
    name: myFunctionApp
    storage_account: myStorageAccount
    app_settings:
      setting1: value1
      setting2: value2

- name: Create container based function app
  azure_rm_functionapp:
    resource_group: myResourceGroup
    name: myFunctionApp
    storage_account: myStorageAccount
    plan:
      resource_group: myResourceGroup
      name: myAppPlan
    container_settings:
      name: httpd
      registry_server_url: index.docker.io

- name: Delete a function app
  azure_rm_functionapp:
    resource_group: myResourceGroup
    name: myFunctionApp
    state: absent
'''

RETURN = '''
state:
    description:
        - Current state of the Azure Function App.
    returned: success
    type: dict
    example:
        id: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Web/sites/myFunctionApp
        name: myfunctionapp
        kind: functionapp
        location: East US
        type: Microsoft.Web/sites
        state: Running
        host_names:
          - myfunctionapp.azurewebsites.net
        repository_site_name: myfunctionapp
        usage_state: Normal
        enabled: true
        enabled_host_names:
          - myfunctionapp.azurewebsites.net
          - myfunctionapp.scm.azurewebsites.net
        availability_state: Normal
        host_name_ssl_states:
          - name: myfunctionapp.azurewebsites.net
            ssl_state: Disabled
            host_type: Standard
          - name: myfunctionapp.scm.azurewebsites.net
            ssl_state: Disabled
            host_type: Repository
        server_farm_id: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Web/serverfarms/EastUSPlan
        reserved: false
        last_modified_time_utc: '2017-08-22T18:54:01.190Z'
        scm_site_also_stopped: false
        client_affinity_enabled: true
        client_cert_enabled: false
        host_names_disabled: false
        outbound_ip_addresses: ............
        container_size: 1536
        daily_memory_time_quota: 0
        resource_group: myResourceGroup
        default_host_name: myfunctionapp.azurewebsites.net
'''  # NOQA

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.web.models import Site, SiteConfig, NameValuePair, ManagedServiceIdentity, UserAssignedIdentity

except ImportError:
    # This is handled in azure_rm_common
    pass

container_settings_spec = dict(
    name=dict(type='str', required=True),
    registry_server_url=dict(type='str'),
    registry_server_user=dict(type='str'),
    registry_server_password=dict(type='str', no_log=True)
)


class AzureRMFunctionApp(AzureRMModuleBaseExt):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True, aliases=['resource_group_name']),
            name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            location=dict(type='str'),
            storage_account=dict(
                type='str',
                aliases=['storage', 'storage_account_name']
            ),
            app_settings=dict(type='dict'),
            plan=dict(
                type='raw'
            ),
            identity=dict(
                type='dict',
                options=self.managed_identity_multiple_spec
            ),
            container_settings=dict(
                type='dict',
                options=container_settings_spec
            )
        )

        self.results = dict(
            changed=False,
            state=dict()
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.location = None
        self.storage_account = None
        self.app_settings = None
        self.plan = None
        self.container_settings = None
        self._managed_identity = None

        # Managed Identity
        self.identity = None

        required_if = [('state', 'present', ['storage_account'])]

        super(AzureRMFunctionApp, self).__init__(
            self.module_arg_spec,
            supports_check_mode=True,
            required_if=required_if
        )

    @property
    def managed_identity(self):
        if not self._managed_identity:
            self._managed_identity = {"identity": ManagedServiceIdentity,
                                      "user_assigned": UserAssignedIdentity
                                      }
        return self._managed_identity

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])
        if self.app_settings is None:
            self.app_settings = dict()

        try:
            resource_group = self.rm_client.resource_groups.get(self.resource_group)
        except Exception:
            self.fail('Unable to retrieve resource group')

        self.location = self.location or resource_group.location

        try:
            function_app = self.web_client.web_apps.get(resource_group_name=self.resource_group, name=self.name)
            # Newer SDK versions (0.40.0+) seem to return None if it doesn't exist instead of raising error
            exists = function_app is not None
        except ResourceNotFoundError as exc:
            exists = False

        if self.state == 'absent':
            if exists:
                if self.check_mode:
                    self.results['changed'] = True
                    return self.results
                try:
                    self.web_client.web_apps.delete(resource_group_name=self.resource_group, name=self.name)
                    self.results['changed'] = True
                except Exception as exc:
                    self.fail('Failure while deleting web app: {0}'.format(exc))
            else:
                self.results['changed'] = False
        else:
            kind = 'functionapp'
            linux_fx_version = None
            if self.container_settings and self.container_settings.get('name'):
                kind = 'functionapp,linux,container'
                linux_fx_version = 'DOCKER|'
                if self.container_settings.get('registry_server_url'):
                    self.app_settings['DOCKER_REGISTRY_SERVER_URL'] = 'https://' + self.container_settings['registry_server_url']
                    linux_fx_version += self.container_settings['registry_server_url'] + '/'
                linux_fx_version += self.container_settings['name']
                if self.container_settings.get('registry_server_user'):
                    self.app_settings['DOCKER_REGISTRY_SERVER_USERNAME'] = self.container_settings.get('registry_server_user')

                if self.container_settings.get('registry_server_password'):
                    self.app_settings['DOCKER_REGISTRY_SERVER_PASSWORD'] = self.container_settings.get('registry_server_password')

            if not self.plan and exists:
                self.plan = function_app.server_farm_id

            curr_identity = function_app.identity.as_dict() if exists and function_app.identity is not None else None
            update_identity = False

            if self.identity:
                update_identity, self.identity = self.update_managed_identity(new_identity=self.identity, curr_identity=curr_identity)

            if not exists:
                function_app = Site(
                    location=self.location,
                    kind=kind,
                    site_config=SiteConfig(
                        app_settings=self.aggregated_app_settings(),
                        scm_type='LocalGit'
                    ),
                    identity=self.identity
                )
                self.results['changed'] = True
            else:
                new_identity = None
                if update_identity:
                    new_identity = self.identity
                self.results['changed'], function_app = self.update(function_app, new_identity)

            # get app service plan
            if self.plan:
                if isinstance(self.plan, dict):
                    self.plan = "/subscriptions/{0}/resourceGroups/{1}/providers/Microsoft.Web/serverfarms/{2}".format(
                        self.subscription_id,
                        self.plan.get('resource_group', self.resource_group),
                        self.plan.get('name')
                    )
                function_app.server_farm_id = self.plan

            # set linux fx version
            if linux_fx_version:
                function_app.site_config.linux_fx_version = linux_fx_version

            if self.check_mode:
                self.results['state'] = function_app.as_dict()
            elif self.results['changed']:
                try:
                    response = self.web_client.web_apps.begin_create_or_update(resource_group_name=self.resource_group,
                                                                               name=self.name,
                                                                               site_envelope=function_app)
                    new_function_app = self.get_poller_result(response)
                    self.results['state'] = new_function_app.as_dict()
                except Exception as exc:
                    self.fail('Error creating or updating web app: {0}'.format(exc))

        return self.results

    def update(self, source_function_app, identity=None):
        """Update the Site object if there are any changes"""

        source_app_settings = self.web_client.web_apps.list_application_settings(
            resource_group_name=self.resource_group,
            name=self.name
        )

        changed, target_app_settings = self.update_app_settings(source_app_settings.properties)

        source_function_app.site_config = SiteConfig(
            app_settings=target_app_settings,
            scm_type='LocalGit'
        )

        if identity is not None:
            source_function_app.identity = identity
            changed = True

        return changed, source_function_app

    def update_app_settings(self, source_app_settings):
        """Update app settings"""

        target_app_settings = self.aggregated_app_settings()
        target_app_settings_dict = dict([(i.name, i.value) for i in target_app_settings])
        return target_app_settings_dict != source_app_settings, target_app_settings

    def necessary_functionapp_settings(self):
        """Construct the necessary app settings required for an Azure Function App"""

        function_app_settings = []

        if self.container_settings is None:
            for key in ['AzureWebJobsStorage', 'WEBSITE_CONTENTAZUREFILECONNECTIONSTRING', 'AzureWebJobsDashboard']:
                function_app_settings.append(NameValuePair(name=key, value=self.storage_connection_string))
            function_app_settings.append(NameValuePair(name='FUNCTIONS_EXTENSION_VERSION', value='~1'))
            function_app_settings.append(NameValuePair(name='WEBSITE_NODE_DEFAULT_VERSION', value='6.5.0'))
            function_app_settings.append(NameValuePair(name='WEBSITE_CONTENTSHARE', value=self.name))
        else:
            function_app_settings.append(NameValuePair(name='FUNCTIONS_EXTENSION_VERSION', value='~2'))
            function_app_settings.append(NameValuePair(name='WEBSITES_ENABLE_APP_SERVICE_STORAGE', value=False))
            function_app_settings.append(NameValuePair(name='AzureWebJobsStorage', value=self.storage_connection_string))

        return function_app_settings

    def aggregated_app_settings(self):
        """Combine both system and user app settings"""

        function_app_settings = self.necessary_functionapp_settings()
        for app_setting_key in self.app_settings:
            found_setting = None
            for s in function_app_settings:
                if s.name == app_setting_key:
                    found_setting = s
                    break
            if found_setting:
                found_setting.value = self.app_settings[app_setting_key]
            else:
                function_app_settings.append(NameValuePair(
                    name=app_setting_key,
                    value=self.app_settings[app_setting_key]
                ))
        return function_app_settings

    @property
    def storage_connection_string(self):
        """Construct the storage account connection string"""

        return 'DefaultEndpointsProtocol=https;AccountName={0};AccountKey={1}'.format(
            self.storage_account,
            self.storage_key
        )

    @property
    def storage_key(self):
        """Retrieve the storage account key"""

        return self.storage_client.storage_accounts.list_keys(
            resource_group_name=self.resource_group,
            account_name=self.storage_account
        ).keys[0].value


def main():
    """Main function execution"""

    AzureRMFunctionApp()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022, Tim Cragg (@timcragg)
# Copyright: (c) 2022, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_aaa_user_role
short_description: Manage AAA user roles (aaa:UserRole)
description:
- Manage AAA User Role configuration on Cisco ACI fabrics.
options:
  aaa_user:
    description:
    - The name of the existing user to add roles and privileges
    type: str
    aliases: [ user_name ]
  aaa_user_type:
    description:
    - Whether this is a normal user or an appuser.
    type: str
    choices: [ appuser, user ]
    default: user
  domain_name:
    description:
    - The name of the security domain
    type: str
    aliases: [ user_domain ]
  name:
    description:
    - Name of the AAA role
    type: str
    aliases: [ role_name, user_role ]
  privilege_type:
    description:
    - Privilege for the role
    type: str
    aliases: [ priv_type ]
    choices: [ read, write ]
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(aaa_user) and C(domain_name) must exist before using this module in your playbook.
  The M(cisco.aci.aci_aaa_user) and M(cisco.aci.aci_aaa_user_domain) modules can be used for this.
seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(aaaUserRole).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
- Sabari Jaganathan (@sajagana)
"""

EXAMPLES = r"""
- name: Add a user role to a user security domain
  cisco.aci.aci_aaa_user_role:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: my_user
    domain_name: my_domain
    name: my_role
    privilege_type: read
    state: present
  delegate_to: localhost

- name: Add list of user roles to a user domain
  cisco.aci.aci_aaa_user_role:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: my_user
    domain_name: my_domain
    name: "{{ item.name }}"
    privilege_type: "{{ item.privilege_type }}"
    state: present
  with_items:
    - name: aaa
      privilege_type: write
    - name: access-admin
      privilege_type: write
    - name: ops
      privilege_type: write
  delegate_to: localhost

- name: Query a user role from a user security domain
  cisco.aci.aci_aaa_user_role:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: my_user
    domain_name: my_domain
    name: my_role
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all user roles from a user security domain
  cisco.aci.aci_aaa_user_role:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: my_user
    domain_name: my_domain
    state: query
  delegate_to: localhost
  register: query_results

- name: Query all user roles from a user
  cisco.aci.aci_aaa_user_role:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: my_user
    state: query
  delegate_to: localhost
  register: query_all_roles_of_user

- name: Query all user roles
  cisco.aci.aci_aaa_user_role:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_all_user_roles

- name: Remove user role from a user domain
  cisco.aci.aci_aaa_user_role:
    host: apic
    username: admin
    password: SomeSecretPassword
    aaa_user: my_user
    domain_name: my_domain
    name: my_role
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec

ACI_MAPPING = dict(
    appuser=dict(
        aci_class="aaaAppUser",
        aci_mo="userext/appuser-",
    ),
    user=dict(
        aci_class="aaaUser",
        aci_mo="userext/user-",
    ),
)

PRIV_TYPE_MAPPING = {
    "read": "readPriv",
    "write": "writePriv",
}


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        aaa_user=dict(type="str", aliases=["user_name"]),
        aaa_user_type=dict(type="str", default="user", choices=["appuser", "user"]),
        domain_name=dict(type="str", aliases=["user_domain"]),
        name=dict(type="str", aliases=["role_name", "user_role"]),
        privilege_type=dict(type="str", aliases=["priv_type"], choices=["read", "write"]),
        name_alias=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["aaa_user", "domain_name", "name"]],
            ["state", "present", ["aaa_user", "domain_name", "name"]],
        ],
    )

    aaa_user = module.params.get("aaa_user")
    aaa_user_type = module.params.get("aaa_user_type")
    domain_name = module.params.get("domain_name")
    name = module.params.get("name")
    privilege_type = module.params.get("privilege_type")
    name_alias = module.params.get("name_alias")
    state = module.params.get("state")

    if privilege_type is not None:
        privilege_type = PRIV_TYPE_MAPPING[privilege_type]

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class=ACI_MAPPING.get(aaa_user_type).get("aci_class"),
            aci_rn="{0}{1}".format(ACI_MAPPING.get(aaa_user_type).get("aci_mo"), aaa_user),
            module_object=aaa_user,
            target_filter={"name": aaa_user},
        ),
        subclass_1=dict(
            aci_class="aaaUserDomain",
            aci_rn="userdomain-{0}".format(domain_name),
            module_object=domain_name,
            target_filter={"name": domain_name},
        ),
        subclass_2=dict(
            aci_class="aaaUserRole",
            aci_rn="role-{0}".format(name),
            module_object=name,
            target_filter={"name": name},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="aaaUserRole",
            class_config=dict(
                name=name,
                privType=privilege_type,
                nameAlias=name_alias,
            ),
        )

        aci.get_diff(aci_class="aaaUserRole")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_postgresqlflexibledatabase
version_added: "2.2.0"
short_description: Manage PostgreSQL Flexible Database instance
description:
    - Create, update and delete instance of PostgreSQL Flexible Database.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource. You can obtain this value from the Azure Resource Manager API or the portal.
        required: True
        type: str
    server_name:
        description:
            - The name of the server.
        required: True
        type: str
    name:
        description:
            - The name of the database.
        required: True
        type: str
    charset:
        description:
            - The charset of the database.
        type: str
    collation:
        description:
            - The collation of the database.
        type: str
    state:
        description:
            - Assert the state of the PostgreSQL Flexible database. Use C(present) to create or update a database and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Create (or update) PostgreSQL Flexible Database
  azure_rm_postgresqlflexibledatabase:
    resource_group: myResourceGroup
    server_name: testserver
    name: db1
    charset: UTF8
    collation: en_US.utf8

- name: Delete PostgreSQL Flexible Database
  azure_rm_postgresqlflexibledatabase:
    resource_group: myResourceGroup
    server_name: testserver
    name: db1
'''

RETURN = '''
database:
    description:
        - A list of dictionaries containing facts for PostgreSQL Flexible Database.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID of the postgresql flexible database.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.DBforPostgreSQL/flexibleServers/postfle9/databases/freddatabase"
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: freddatabase
        charset:
            description:
                - The charset of the database.
            returned: always
            type: str
            sample: UTF-8
        collation:
            description:
                - The collation of the database.
            returned: always
            type: str
            sample: en_US.utf8
        type:
            description:
                - The type of the resource.
            returned: always
            type: str
            sample: Microsoft.DBforPostgreSQL/flexibleServers/databases
'''


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMPostgreSqlFlexibleDatabases(AzureRMModuleBase):
    """Configuration class for an Azure RM PostgreSQL Flexible Database resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            server_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            charset=dict(
                type='str'
            ),
            collation=dict(
                type='str'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.server_name = None
        self.name = None
        self.parameters = dict()

        self.results = dict(changed=False)
        self.state = None

        super(AzureRMPostgreSqlFlexibleDatabases, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                                 supports_check_mode=True,
                                                                 supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                if key == "charset":
                    self.parameters["charset"] = kwargs[key]
                elif key == "collation":
                    self.parameters["collation"] = kwargs[key]

        old_response = None
        response = None
        changed = False

        old_response = self.get_postgresqlflexibledatabase()

        if not old_response:
            self.log("PostgreSQL Flexible Database instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                changed = True
                if not self.check_mode:
                    response = self.create_update_postgresqlflexibledatabase(self.parameters)
        else:
            self.log("PostgreSQL Flexible Database instance already exists")
            if self.state == 'absent':
                changed = True
                if not self.check_mode:
                    response = self.delete_postgresqlflexibledatabase()
            else:
                if (self.parameters.get('charset') is not None and self.parameters['charset'] != old_response['charset']) or\
                        (self.parameters.get('collation') is not None and self.parameters['collation'] != old_response['collation']):
                    changed = True
                    if not self.check_mode:
                        self.fail("The Post Gresql Flexible database not support to update")
                else:
                    response = old_response

        self.results['database'] = response
        self.results['changed'] = changed
        return self.results

    def create_update_postgresqlflexibledatabase(self, body):
        '''
        Creates or updates PostgreSQL Flexible Database with the specified configuration.

        :return: deserialized PostgreSQL Flexible Database instance state dictionary
        '''
        self.log("Creating / Updating the PostgreSQL Flexible Database instance {0}".format(self.name))

        try:
            response = self.postgresql_flexible_client.databases.begin_create(resource_group_name=self.resource_group,
                                                                              server_name=self.server_name,
                                                                              database_name=self.name,
                                                                              parameters=body)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the PostgreSQL Flexible Database instance.')
            self.fail("Error creating the PostgreSQL Flexible Database instance: {0}".format(str(exc)))
        return self.format_item(response)

    def delete_postgresqlflexibledatabase(self):
        '''
        Deletes specified PostgreSQL Flexible Database instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the PostgreSQL Flexible Database instance {0}".format(self.name))
        try:
            self.postgresql_flexible_client.databases.begin_delete(resource_group_name=self.resource_group,
                                                                   server_name=self.server_name,
                                                                   database_name=self.name)
        except Exception as ec:
            self.log('Error attempting to delete the PostgreSQL Flexible Database instance.')
            self.fail("Error deleting the PostgreSQL Flexible Database instance: {0}".format(str(ec)))

    def get_postgresqlflexibledatabase(self):
        '''
        Gets the properties of the specified PostgreSQL Flexible Database.

        :return: deserialized PostgreSQL Flexible Database instance state dictionary
        '''
        self.log("Checking if the PostgreSQL Flexible Database instance {0} is present".format(self.name))
        found = False
        try:
            response = self.postgresql_flexible_client.databases.get(resource_group_name=self.resource_group,
                                                                     server_name=self.server_name,
                                                                     database_name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("PostgreSQL Flexible Database instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the PostgreSQL Flexible Database instance. Exception as {0}'.format(e))
        if found is True:
            return self.format_item(response)

        return None

    def format_item(self, item):
        result = dict(
            id=item.id,
            name=item.name,
            type=item.type,
            charset=item.charset,
            collation=item.collation
        )
        return result


def main():
    """Main execution"""
    AzureRMPostgreSqlFlexibleDatabases()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_query_count_v1
short_description: Resource module for Network Devices Query Count V1
description:
- Manage operation create of the resource Network Devices Query Count V1.
- >
   Gets the total number Network Devices based on the provided complex filters and aggregation functions. For
   detailed information about the usage of the API, please refer to the Open API specification document - https
   //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
   AssuranceNetworkDevices-1.0.2-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Network Devices Query Count's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Network Devices Query Count's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: str
    type: list
  page:
    description: Network Devices Query Count's page.
    suboptions:
      limit:
        description: Limit.
        type: int
      offset:
        description: Offset.
        type: int
      sortBy:
        description: Network Devices Query Count's sortBy.
        elements: dict
        suboptions:
          name:
            description: Name.
            type: str
          order:
            description: Order.
            type: str
        type: list
    type: dict
  startTime:
    description: Start Time.
    type: int
  views:
    description: Views.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetsTheTotalNumberNetworkDevicesBasedOnTheProvidedComplexFiltersAndAggregationFunctionsV1  # noqa: E501
  description: Complete reference of the GetsTheTotalNumberNetworkDevicesBasedOnTheProvidedComplexFiltersAndAggregationFunctionsV1 API.  # noqa: E501
  link: https://developer.cisco.com/docs/dna-center/#!gets-the-total-number-network-devices-based-on-the-provided-complex-filters-and-aggregation-functions  # noqa: E501
notes:
  - SDK Method used are
    devices.Devices.gets_the_total_number_network_devices_based_on_the_provided_complex_filters_and_aggregation_functions_v1,  # noqa: E501

  - Paths used are
    post /data/api/v1/networkDevices/query/count,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.network_devices_query_count_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
    - function: string
      name: string
    attributes:
    - string
    endTime: 0
    filters:
    - key: string
      operator: string
      value: string
    page:
      limit: 0
      offset: 0
      sortBy:
      - name: string
        order: string
    startTime: 0
    views:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-

# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c), Michael DeHaan <michael.dehaan@gmail.com>, 2012-2013
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from functools import cmp_to_key

from ansible.module_utils._text import to_text
from ansible.module_utils.six import binary_type
from ansible.module_utils.six import string_types


def _canonify_root_arn(arn):
    # There are multiple ways to specifiy delegation of access to an account
    # https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_elements_principal.html#principal-accounts
    if arn.startswith("arn:aws:iam::") and arn.endswith(":root"):
        arn = arn.split(":")[4]
    return arn


def _canonify_policy_dict_item(item, key):
    """
    Converts special cases where there are multiple ways to write the same thing into a single form
    """
    # There are multiple ways to specify anonymous principals
    # https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_elements_principal.html#principal-anonymous
    if key in ["NotPrincipal", "Principal"]:
        if item == "*":
            return {"AWS": "*"}
    return item


def _tuplify_list(element):
    if isinstance(element, list):
        return tuple(element)
    return element


def _hashable_policy(policy, policy_list):
    """
    Takes a policy and returns a list, the contents of which are all hashable and sorted.
    Example input policy:
    {'Version': '2012-10-17',
     'Statement': [{'Action': 's3:PutObjectAcl',
                    'Sid': 'AddCannedAcl2',
                    'Resource': 'arn:aws:s3:::test_policy/*',
                    'Effect': 'Allow',
                    'Principal': {'AWS': ['arn:aws:iam::XXXXXXXXXXXX:user/username1', 'arn:aws:iam::XXXXXXXXXXXX:user/username2']}
                   }]}
    Returned value:
    [('Statement',  ((('Action', ('s3:PutObjectAcl',)),
                      ('Effect', ('Allow',)),
                      ('Principal', ('AWS', (('arn:aws:iam::XXXXXXXXXXXX:user/username1',), ('arn:aws:iam::XXXXXXXXXXXX:user/username2',)))),
                      ('Resource', ('arn:aws:s3:::test_policy/*',)), ('Sid', ('AddCannedAcl2',)))),
     ('Version', ('2012-10-17',)))]

    """
    # Amazon will automatically convert bool and int to strings for us
    if isinstance(policy, bool):
        return tuple([str(policy).lower()])
    elif isinstance(policy, int):
        return tuple([str(policy)])

    if isinstance(policy, list):
        for each in policy:
            hashed_policy = _hashable_policy(each, [])
            tupleified = _tuplify_list(hashed_policy)
            policy_list.append(tupleified)
    elif isinstance(policy, string_types) or isinstance(policy, binary_type):
        policy = to_text(policy)
        # convert root account ARNs to just account IDs
        policy = _canonify_root_arn(policy)
        return [policy]
    elif isinstance(policy, dict):
        # Sort the keys to ensure a consistent order for later comparison
        sorted_keys = list(policy.keys())
        sorted_keys.sort()
        for key in sorted_keys:
            # Converts special cases to a consistent form
            element = _canonify_policy_dict_item(policy[key], key)
            hashed_policy = _hashable_policy(element, [])
            tupleified = _tuplify_list(hashed_policy)
            policy_list.append((key, tupleified))
    # ensure we aren't returning deeply nested structures of length 1
    if len(policy_list) == 1 and isinstance(policy_list[0], tuple):
        policy_list = policy_list[0]
    if isinstance(policy_list, list):
        policy_list.sort(key=cmp_to_key(_py3cmp))
    return policy_list


def _py3cmp(a, b):
    """Python 2 can sort lists of mixed types. Strings < tuples. Without this function this fails on Python 3."""
    try:
        if a > b:
            return 1
        elif a < b:
            return -1
        else:
            return 0
    except TypeError as e:
        # check to see if they're tuple-string
        # always say strings are less than tuples (to maintain compatibility with python2)
        str_ind = to_text(e).find("str")
        tup_ind = to_text(e).find("tuple")
        if -1 not in (str_ind, tup_ind):
            if str_ind < tup_ind:
                return -1
            elif tup_ind < str_ind:
                return 1
        raise


def compare_policies(current_policy, new_policy, default_version="2008-10-17"):
    """Compares the existing policy and the updated policy
    Returns True if there is a difference between policies.
    """
    # https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_elements_version.html
    if default_version:
        if isinstance(current_policy, dict):
            current_policy = current_policy.copy()
            current_policy.setdefault("Version", default_version)
        if isinstance(new_policy, dict):
            new_policy = new_policy.copy()
            new_policy.setdefault("Version", default_version)

    return set(_hashable_policy(new_policy, [])) != set(_hashable_policy(current_policy, []))
